# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from copy import deepcopy
from enum import Enum
from typing import List, Dict, Any

from pycivil.EXAUtils.EXAExceptions import EXAExceptions as Ex
from pycivil.EXAStructural.codes import Code
from pycivil.EXAStructural.loads import ForcesOnSection


class Checkable:
    def __init__(self):
        self.__results = {}
        self.__ck = None
        self.__criteria = None
        self.__criteriaForCheck = []
        self.__code = None
        self.__loads = None
        self.__loadsSelector = None

    def _getResults(self):
        return self.__results

    def _setCode(self, code: Code) -> None:
        if not isinstance(code, Code):
            raise Ex("002", "code must be a Code type", type(code))
        self.__code = code

    def _setLoads(self, loads: List[ForcesOnSection]) -> None:
        if not isinstance(loads, list):
            raise Ex("001", "loads must be a list type", type(loads))
        self.__loads = loads

    def _setCriteria(self, criteria: List[str]) -> None:
        self.__criteria = criteria

    def _setCheckableObj(self, obj):
        self.__ck = obj

    def _setLoadsSelector(self, s):
        if not isinstance(s, list):
            raise Ex("001", "s must be a list type", type(s))
        self.__loadsSelector = s

    def _validateCriteria(self, criteria: List[str]) -> None:
        if not isinstance(criteria, list):
            raise Ex("0001", "criteria must be a List[str]", type(criteria))

        for v in criteria:
            if not isinstance(v, str):
                raise Ex("0001", "criteria must be a List[str]", type(v))
            assert self.__criteria is not None
            if v not in self.__criteria:
                raise Ex("0001", "value is not in criteria", v)

        self.__criteriaForCheck = criteria

    def _ck(self):
        return self.__ck

    def _criteriaForCheck(self):
        return self.__criteriaForCheck

    def check(
        self,
        criteria: List[str],
        loads: List[ForcesOnSection],
        law: Code,
        loadSelector: List[List[int]],
    ) -> Dict[Any, Any]:
        raise NotImplementedError("Need to be implemented")

    def getResults(self):
        return deepcopy(self.__results)


class Checker:
    def __init__(self):
        self.__checkable = None
        self.__results: Dict[Any, Any] | None = None

    def _setResults(self, r):
        if isinstance(r, dict):
            self.__results = r
        else:
            raise TypeError("arguments must be a dict type")

    def check(
        self,
        criteria: List[str],
        loads: List[ForcesOnSection],
        law: Code,
        loadsSelector: List[List[int]] | None = None,
    ) -> None:
        if self.__checkable is None:
            raise ValueError("Before launch check need to use setCheckable() method")
        if loadsSelector is None:
            loadsSelector = []
        self.__results = self.__checkable.check(criteria, loads, law, loadsSelector)

    def setCheckable(self, c: Checkable) -> None:
        if not isinstance(c, Checkable):
            raise TypeError("arguments must be a checkable")

        self.__checkable = c

    def getResults(self) -> Dict[Any, Any] | None:
        return self.__results


class CheckableCriteriaEnum(str, Enum):
    SLE_NM = "SLE-NM"
    SLE_F = "SLE-F"
    SLU_NM = "SLU-NM"
    SLU_T = "SLU-T"
    SLU_NM_FIRE = "SLU-NM-FIRE"



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import glob
import json
import os
import uuid
from enum import Enum
from typing import Any, Dict, List, Union, Literal

import vtkmodules.all as vtk

# Node3d is point with id
from pycivil.EXAGeometry.geometry import Node3d, Point3d, areaFromTria3D
from pycivil.EXAUtils.logging import log


class FileType(Enum):
    VTM_TYPE = 0
    VTU_TYPE = 1


class Parser:
    def __init__(self) -> None:
        self.__currentFile: int = 0
        self.__files: List[str] = []
        self.__filesSuffix: str = ""
        self.__dataParsed: Any = {}
        self.__hashTable_Blocks: Dict[str, int] = {}
        self.__type: FileType = FileType.VTM_TYPE

    def parse(
        self, files: str = "", type: FileType = FileType.VTM_TYPE, ll: Literal[0, 1, 2, 3] = 1
    ) -> bool:
        # self.__type = type
        return self.__parse__(files, ll)

    def setFileType(self, type: FileType) -> None:
        self.__type = type

    def __parse__(self, files: str = "", ll: Literal[0, 1, 2, 3] = 1) -> bool:

        if self.__type == FileType.VTM_TYPE:
            self.__filesSuffix = ".vtm"

        elif self.__type == FileType.VTU_TYPE:
            self.__filesSuffix = ".vtu"

        else:
            log(
                "ERR",
                "File type UNKNOWN !!!",
                ll,
            )
            return False

        if os.path.exists(files):
            if os.path.isfile(files):
                self.__files = [files]
            else:
                if os.path.isdir(files):
                    filesList = glob.glob(
                        os.path.join(
                            os.path.expanduser(files), "*" + self.__filesSuffix
                        )
                    )
                    if len(filesList) > 0:
                        self.__files = filesList
                    else:
                        log(
                            "ERR",
                            "None file in path <{}> with suffix <{}> !!!".format(
                                files, self.__filesSuffix
                            ),
                            ll,
                        )
                        return False
                else:
                    log("ERR", "Files arg is not file and dir !!!", ll)
                    return False
        else:
            log("ERR", "Files arg file and dir dont exists !!!", ll)
            return False

        log("INF", "Files charged: \n{}".format("\n".join(self.__files)), ll)
        log("INF", f"Current is *{self.__files[self.__currentFile]}*", ll)

        dataDict: Any = {}
        for f in self.__files:
            dataDict[f] = {"ReaderData": {}}

            log("INF", "*" * len(f), ll)
            log("INF", f, ll)
            log("INF", "*" * len(f), ll)

            if self.__type == FileType.VTM_TYPE:
                reader = vtk.vtkXMLMultiBlockDataReader()  # type: ignore

            if self.__type == FileType.VTU_TYPE:
                reader = vtk.vtkXMLUnstructuredGridReader()  # type: ignore

            reader.SetFileName(f)
            reader.Update()

            log("INF", "**************", ll)
            log("INF", "PARSING READER", ll)
            log("INF", "**************", ll)

            log(
                "INF",
                f"Arrays by cells  number is: {reader.GetNumberOfCellArrays()}",
                ll,
            )
            dataDict[f]["ReaderData"]["CellArrayName"] = []
            for i in range(reader.GetNumberOfCellArrays()):
                dataDict[f]["ReaderData"]["CellArrayName"].append(
                    reader.GetCellArrayName(i)
                )
                log(
                    "INF",
                    f'Array nb.{i} name --> "{reader.GetCellArrayName(i)}"',
                    ll,
                )

            log(
                "INF",
                "Arrays by points number is : {}".format(
                    reader.GetNumberOfPointArrays()
                ),
                ll,
            )
            dataDict[f]["ReaderData"]["PointArrayName"] = []
            for i in range(reader.GetNumberOfPointArrays()):
                dataDict[f]["ReaderData"]["PointArrayName"].append(
                    reader.GetPointArrayName(i)
                )
                log(
                    "INF",
                    f'Array nb.{i} nome --> "{reader.GetPointArrayName(i)}"',
                    ll,
                )

            dataDict[f]["DataSet"] = {}
            log(
                "INF",
                f"Output ports number is: {reader.GetNumberOfOutputPorts()}",
                ll,
            )
            dataSet = reader.GetOutput()

            log("INF", "***************", ll)
            log("INF", "PARSING DATASET", ll)
            log("INF", "***************", ll)
            log(
                "INF",
                f"Cells  number in dataset: {dataSet.GetNumberOfCells()}",
                ll,
            )
            log(
                "INF",
                f"Points number in dataset: {dataSet.GetNumberOfPoints()}",
                ll,
            )
            if self.__type == FileType.VTM_TYPE:
                log(
                    "INF",
                    f"Block number in dataser: {dataSet.GetNumberOfBlocks()}",
                    ll,
                )
                dataDict[f]["DataSet"]["NumberOfBlocks"] = dataSet.GetNumberOfBlocks()

            if self.__type == FileType.VTU_TYPE:
                log(
                    "INF",
                    "The dataset isn't multiblock. Nb of blocks setted to 0.",
                    ll,
                )
                dataDict[f]["DataSet"]["NumberOfBlocks"] = 0

            dataDict[f]["DataSet"]["NumberOfCells"] = dataSet.GetNumberOfCells()
            dataDict[f]["DataSet"]["NumberOfPoints"] = dataSet.GetNumberOfPoints()

            fieldData = dataSet.GetFieldData()
            log(
                "INF",
                "Arrays by fieldData number is : {}".format(
                    fieldData.GetNumberOfArrays()
                ),
                ll,
            )
            dataDict[f]["FieldData"] = {}
            space = " " * 1
            for i in range(fieldData.GetNumberOfArrays()):
                dataDict[f]["FieldData"][fieldData.GetArrayName(i)] = []
                log(
                    "INF",
                    space * 2
                    + 'Arrays fieldData nb.{} name --> "{}"'.format(
                        i, fieldData.GetArrayName(i)
                    ),
                    ll,
                )
                fieldArray = fieldData.GetArray(i)
                log(
                    "INF",
                    space * 4
                    + "Tuples array fieldData {}".format(
                        fieldArray.GetNumberOfTuples()
                    ),
                    ll,
                )
                log(
                    "INF",
                    space * 4
                    + "Components array fieldData {}".format(
                        fieldArray.GetNumberOfComponents()
                    ),
                    ll,
                )
                for t in range(fieldArray.GetNumberOfTuples()):
                    log(
                        "INF",
                        space * 6 + f"Tuple n.{t} value --> {fieldArray.GetTuple(t)}",
                        ll,
                    )

                    dataDict[f]["FieldData"][fieldData.GetArrayName(i)].append({})
                    for c in range(fieldArray.GetNumberOfComponents()):
                        dataDict[f]["FieldData"][fieldData.GetArrayName(i)][t][
                            fieldArray.GetComponentName(c)
                        ] = fieldArray.GetComponent(t, c)
                        log(
                            "INF",
                            space * 8
                            + "Component nb.{} value --> {}".format(
                                c, fieldArray.GetComponent(t, c)
                            ),
                            ll,
                        )

            if self.__type == FileType.VTM_TYPE:
                dataDict[f]["Blocks"] = []
                for b in range(dataSet.GetNumberOfBlocks()):
                    log("INF", space * 2 + f"Add block nb.{b} ", ll)
                    uGrid = dataSet.GetBlock(b)
                    uuidForTable = str(uuid.uuid4())
                    self.__hashTable_Blocks[uuidForTable] = uGrid
                    dataDict[f]["Blocks"].append(uuidForTable)

            if self.__type == FileType.VTU_TYPE:
                log("INF", space * 2 + "Add grid ", ll)
                uGrid = dataSet
                uuidForTable = str(uuid.uuid4())
                self.__hashTable_Blocks[uuidForTable] = uGrid
                dataDict[f]["Grid"] = uuidForTable

        log("INF", json.dumps(dataDict, indent=2), ll)
        self.__dataParsed = dataDict
        return True

    # First occorrence for the key
    def findMultiblockKeyByArrayValue(
        self,
        arrayName: str,
        value: float,
        tuple: int = 0,
        componentName: Union[str, None] = None,
        ll: Literal[0, 1, 2, 3] = 0,
    ) -> str:
        if len(self.__dataParsed) == 0:
            log("ERR", "Data are not parsed !!!", ll)
            return ""
        for k in self.__dataParsed.keys():
            try:
                v = self.__dataParsed[k]["FieldData"][arrayName][tuple][componentName]
                if v == value:
                    log("INF", f"Value {v} found", ll)
                    return k
            except KeyError:
                log("ERR", "ERROR: fetching key !!!", ll)

        log("INF", f"WARNING: Value {value} not found !!!", ll)
        return ""

    def setCurrentMultiblockByKey(self, key: str, ll: Literal[0, 1, 2, 3] = 1) -> bool:
        idx = self.__currentFile
        try:
            idx = self.__files.index(key)
        except ValueError:
            log(
                "ERR",
                f"setCurrentMultiblockByKey(): Key <{key}> not found !!!",
                ll,
            )
            return False

        log("INF", f"Current multiblock changed to {key}", ll)
        self.__currentFile = idx
        return True

    def setCurrentFileByIndex(self, idx: int, ll: Literal[0, 1, 2, 3] = 1) -> bool:
        if idx >= 0 and idx < len(self.__files):
            for i in range(len(self.__files)):
                if f"-{str(idx)}.vtu" in self.__files[i]:
                    self.__currentFile = i
                    log("INF", f"File selected by index*{self.__files[i]}*", ll)
                    return True

            log("ERR", f"Wrong index {idx}. Index not in files ", ll)
            return False

        log("ERR", f"Wrong index {idx}. Index must be in [{0,len(self.__files)}]", ll)
        return False

    def __mesh(self, block: int = 0) -> Any:

        if self.__type == FileType.VTM_TYPE:
            blockId = self.__currentFile
            blockKey = self.__files[blockId]
            uuid = self.__dataParsed[blockKey]["Blocks"][block]
            return self.__hashTable_Blocks[uuid]

        if self.__type == FileType.VTU_TYPE:
            blockKey = self.__files[self.__currentFile]
            uuid = self.__dataParsed[blockKey]["Grid"]
            return self.__hashTable_Blocks[uuid]

        return None

    def getPoints(self, block: int = 0, ll: int = 3) -> List[Node3d]:
        uGrid = self.__mesh(block)
        nodes = []
        for i in range(uGrid.GetNumberOfPoints()):
            point = uGrid.GetPoint(i)
            nodes.append(Node3d(point[0], point[1], point[2], i))
        return nodes

    def getPoint(self, id: int, block: int = 0, ll: Literal[0, 1, 2, 3] = 3) -> Node3d:
        uGrid = self.__mesh(block)
        if 0 <= id and id <= uGrid.GetNumberOfPoints():
            return Node3d(
                uGrid.GetPoint(id)[0], uGrid.GetPoint(id)[1], uGrid.GetPoint(id)[2], id
            )
        else:
            log("ERR", "Point id out of range", ll)
            return Node3d()

    def getPointValueById(
        self,
        id: int,
        arrayName: str,
        block: int = 0,
        componentIdx: int = 0,
        ll: Literal[0, 1, 2, 3] = 1,
    ) -> Union[float, None]:
        uGrid = self.__mesh(block)
        # Point data == 0
        # Cell data == 1
        pointData = uGrid.GetAttributes(0)
        array = pointData.GetArray(arrayName)

        if array is None:
            log(
                "ERR",
                f"Array name *{arrayName}* do not exists and return None !!!",
                ll,
            )
            return None

        return array.GetComponent(id, componentIdx)

    def getNearestPoint(self, p: Point3d, block: int = 0, ll: Literal[0, 1, 2, 3] = 3) -> Node3d:
        pointLocator = vtk.vtkPointLocator()  # type: ignore
        uGrid = self.__mesh(block)
        pointLocator.SetDataSet(uGrid)
        pointId = pointLocator.FindClosestPoint(p.x, p.y, p.z)
        point = uGrid.GetPoint(pointId)
        return Node3d(point[0], point[1], point[2], pointId)

    def getPointsCellNearestPoint(
        self, p: Point3d, block: int = 0, ll: Literal[0, 1, 2, 3] = 3
    ) -> List[Node3d]:
        cellLocator = vtk.vtkCellLocator()  # type: ignore
        uGrid = self.__mesh(block)
        cellLocator.SetDataSet(uGrid)
        cellLocator.BuildLocator()
        cellId = cellLocator.FindCell([p.x, p.y, p.z])
        if cellId == -1:
            log("INF", "Cell id not found", ll)
            return []

        log("INF", f"Cell id found *{cellId}*", ll)
        cell = uGrid.GetCell(cellId)
        pl = []
        for i in range(cell.GetNumberOfPoints()):
            idPoint = cell.GetPointIds().GetId(i)
            xcoord = cell.GetPoints().GetPoint(i)[0]
            ycoord = cell.GetPoints().GetPoint(i)[1]
            zcoord = cell.GetPoints().GetPoint(i)[2]
            pl.append(Node3d(xcoord, ycoord, zcoord, idPoint))

        for p in pl:
            log("INF", f"--> {p}", ll)
        return pl

    def getPointValueInterpTriaByIds(
        self,
        p: Point3d,
        id0: int,
        id1: int,
        id2: int,
        arrayName: str,
        block: int = 0,
        componentIdx: int = 0,
        ll: Literal[0, 1, 2, 3] = 3,
    ) -> Union[float, None]:

        P0 = self.getPoint(id0, block, ll)
        V0 = self.getPointValueById(
            id=id0, arrayName=arrayName, block=block, componentIdx=componentIdx
        )
        if V0 is None:
            return None
        P1 = self.getPoint(id1, block, ll)
        V1 = self.getPointValueById(
            id=id1, arrayName=arrayName, block=block, componentIdx=componentIdx
        )
        if V1 is None:
            return None
        P2 = self.getPoint(id2, block, ll)
        V2 = self.getPointValueById(
            id=id2, arrayName=arrayName, block=block, componentIdx=componentIdx
        )
        if V2 is None:
            return None

        a0 = areaFromTria3D(p, P1, P2) / areaFromTria3D(P0, P1, P2)
        a1 = areaFromTria3D(p, P0, P2) / areaFromTria3D(P0, P1, P2)
        a2 = areaFromTria3D(p, P0, P1) / areaFromTria3D(P0, P1, P2)

        return a0 * V0 + a1 * V1 + a2 * V2



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pathlib import Path
import pytest

import numpy as np
import pandas as pd
from pycivil.EXAUtils.report import (
    Fragment,
    Reporter,
    ReportTemplateEnum,
    getTemplatesPath,
)

@pytest.mark.needLatex
def test_report_latex_koma_01(tmp_path: Path):
    # Test per documenti latex
    latexTemplatePath = getTemplatesPath()

    f1 = Fragment(latexTemplatePath)
    f1.add("Hello world EN")

    f2 = Fragment(latexTemplatePath)
    f2.add("Ciao mondo IT")

    reporter = Reporter(latexTemplatePath)
    reporter.linkFragments(template=ReportTemplateEnum.TEX_KOMA, fragments=[f1, f2])

    reporter.compileDocument(path=str(tmp_path))
    assert tmp_path.joinpath("report.pdf").exists()
    assert tmp_path.joinpath("report.tex").exists()



@pytest.mark.needLatex
def test_report_latex_koma_02(tmp_path: Path):
    """Latex PDF document is created successfully."""
    f1 = Fragment(latexTemplatePath=getTemplatesPath())
    f1.add("Hello world EN")

    f2 = Fragment(latexTemplatePath=getTemplatesPath())
    f2.add(r"\textbf{Ciao mondo IT}")

    random_data = pd.DataFrame(np.random.random((5, 5)))
    f2.add(random_data.to_latex())

    reporter = Reporter(latexTemplatePath=getTemplatesPath())
    reporter.linkFragments(template=ReportTemplateEnum.TEX_KOMA, fragments=[f1, f2])

    reporter.compileDocument(path=str(tmp_path))
    assert tmp_path.joinpath("report.pdf").exists()
    assert tmp_path.joinpath("report.tex").exists()



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

"""Module that generates Cheat Sheets.

A cheat sheet is a synthetic view of something, of objects with a lots of
properties. In this case, for example PlateMinimumAreaCS show input data,
calculations and output in synthetic manner.

Classes list:

    1. BeamMinimumAreaCS
    2. ConcreteMaterialCS
    3. PlateMinimumAreaCS
    4. RebarMaterialCS
"""

from pathlib import Path

from pycivil.EXAStructural.cheatsheets.codeEC2Rules import PlateMinimumArea
from pycivil.EXAStructural.cheatsheets.codeNTC2018Rules import (
    BeamMinimumArea,
    ConcreteMaterial,
    RebarMaterial,
)
from pycivil.EXAUtils.report import Fragment, FragmentsBuilder


class ConcreteMaterialCS(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, data: ConcreteMaterial) -> None:
        super().__init__()
        self.__latexTemplatePath = latexTemplatePath
        self.__data = data

    def setDataModel(self, data: ConcreteMaterial) -> None:
        self.__data = data

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        placeHolders = {
            "elementDescr": f"{self.__data.inputData.elementDescr:s}",
            "keyCode": f"{self.__data.inputData.keyCode:s}",
            "concreteClass": f"{self.__data.inputData.concreteClass:s}",
            "value_Rck": f"{self.__data.outputData.value_Rck:.1f}",
            "value_fck": f"{self.__data.outputData.value_fck:.1f}",
            "value_fcm": f"{self.__data.outputData.value_fcm:.2f}",
            "value_fctm": f"{self.__data.outputData.value_fctm:.2f}",
            "value_Ecm": f"{self.__data.outputData.value_Ecm:.2f}",
            "value_alphacc": f"{self.__data.outputData.value_alphacc:.2f}",
            "value_gammac": f"{self.__data.outputData.value_gammac:.2f}",
            "value_fcd": f"{self.__data.outputData.value_fcd:.2f}",
            "value_sigmaCar": f"{self.__data.outputData.value_sigmaCar:.2f}",
            "value_sigmaQp": f"{self.__data.outputData.value_sigmaQp:.2f}",
        }
        f.add(
            templateName="sheet-MatConcrete-NTC2018-ita.tex",
            templatePlaceholders=placeHolders,
        )
        self._setFragment(f)
        return f


class RebarMaterialCS(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, data: RebarMaterial) -> None:
        super().__init__()
        self.__latexTemplatePath = latexTemplatePath
        self.__data = data

    def setDataModel(self, data: RebarMaterial) -> None:
        self.__data = data

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        placeHolders = {
            "elementDescr": f"{self.__data.inputData.elementDescr:s}",
            "keyCode": f"{self.__data.inputData.keyCode:s}",
            "steelClass": f"{self.__data.inputData.steelClass:s}",
            "value_fyk": f"{self.__data.outputData.value_fyk:.1f}",
            "value_ftk": f"{self.__data.outputData.value_ftk:.1f}",
            "value_gammas": f"{self.__data.outputData.value_gammas:.2f}",
            "value_Es": f"{self.__data.outputData.value_Es:.2f}",
            "value_fyd": f"{self.__data.outputData.value_fyd:.2f}",
            "value_sigmaCar": f"{self.__data.outputData.value_sigmaCar:.2f}",
        }
        f.add(
            templateName="sheet-MatRebar-NTC2018-ita.tex",
            templatePlaceholders=placeHolders,
        )
        self._setFragment(f)
        return f


class PlateMinimumAreaCS(FragmentsBuilder):
    """The class builds a fragment report starting from PlateMinimumArea class

    The class needs a Reporter class in Report module to build real pdf file.

    Examples:

    ```
    fragBuilder = latexCheatSheets.PlateMinimumAreaCS(
        latexTemplatePath=latexTemplatePath,
        data=data
    )

    frag_title = Fragment()
    frag_title.add(line=r"\section{Area minima delle piastre secondo EC2}")
    frag = fragBuilder.buildFragment()

    reporter = Reporter(latexTemplatePath)
    reporter.buildPDF(ReportDriverEnum.PDFLATEX, ReportTemplateEnum.TEX_ENG_CAL, [frag_title,frag])
    reporter.makePDF(path=str(tmp_path), fileName="TEX_ENG_CAL")
    ```
    """
    def __init__(self, latexTemplatePath: Path, data: PlateMinimumArea) -> None:
        super().__init__()
        self.__latexTemplatePath = latexTemplatePath
        self.__data = data

    def setDataModel(self, data: PlateMinimumArea) -> None:
        self.__data = data

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        placeHolders = {
            "elementDescr": f"{self.__data.inputData.elementDescr:s}",
            "keyCode": f"{self.__data.inputData.keyCode:s}",
            "concreteClass": f"{self.__data.inputData.concreteClass:s}",
            "cls_fck": f"{self.__data.logsData.cls_fck:.2f}",
            "cls_fctm": f"{self.__data.logsData.cls_fctm:.2f}",
            "steelClass": f"{self.__data.inputData.steelClass:s}",
            "steel_fyk": f"{self.__data.logsData.steel_fyk:.2f}",
            "hEl": f"{self.__data.inputData.hEl:.2f}",
            "rebarD": f"{self.__data.inputData.rebarD:.2f}",
            "rebarDSec": f"{self.__data.inputData.rebarDSec:.2f}",
            "cover": f"{self.__data.inputData.cover:.2f}",
            "coverSec": f"{self.__data.inputData.coverSec:.2f}",
            "stirrupD": f"{self.__data.inputData.stirrupD:.2f}",
            "nbLegDirX": f"{self.__data.inputData.nbLegDirX:.2f}",
            "heightUtil": f"{self.__data.logsData.heightUtil:.2f}",
            "areaUtil": f"{self.__data.logsData.areaUtil:.2f}",
            "minimumRebarAreaCrit1": f"{self.__data.logsData.minimumRebarAreaCrit1:.2f}",
            "minimumRebarAreaCrit2": f"{self.__data.logsData.minimumRebarAreaCrit2:.2f}",
            "minimumRebarArea": f"{self.__data.logsData.minimumRebarArea:.2f}",
            "distMaxRebar": f"{self.__data.logsData.distMaxRebar:.2f}",
            "distMaxRebarMaxLoad": f"{self.__data.logsData.distMaxRebarMaxLoad:.2f}",
            "disposedRebarArea": f"{self.__data.logsData.disposedRebarArea:.2f}",
            "disposedRebarNumber": f"{self.__data.logsData.disposedRebarNumber:.2f}",
            "disposedRebarAreaMaxLoad": f"{self.__data.logsData.disposedRebarAreaMaxLoad:.2f}",
            "disposedRebarNumberMaxLoad": f"{self.__data.logsData.disposedRebarNumberMaxLoad:.2f}",
            "heightUtilSec": f"{self.__data.logsData.heightUtilSec:.2f}",
            "areaUtilSec": f"{self.__data.logsData.areaUtilSec:.2f}",
            "minimumRebarAreaCrit1Sec": f"{self.__data.logsData.minimumRebarAreaCrit1Sec:.2f}",
            "minimumRebarAreaCrit2Sec": f"{self.__data.logsData.minimumRebarAreaCrit2Sec:.2f}",
            "minimumRebarAreaSec": f"{self.__data.logsData.minimumRebarAreaSec:.2f}",
            "distMaxRebarSec": f"{self.__data.logsData.distMaxRebarSec:.2f}",
            "distMaxRebarMaxLoadSec": f"{self.__data.logsData.distMaxRebarMaxLoadSec:.2f}",
            "disposedRebarAreaSec": f"{self.__data.logsData.disposedRebarAreaSec:.2f}",
            "disposedRebarNumberSec": f"{self.__data.logsData.disposedRebarNumberSec:.2f}",
            "disposedRebarAreaMaxLoadSec": f"{self.__data.logsData.disposedRebarAreaMaxLoadSec:.2f}",
            "disposedRebarNumberMaxLoadSec": f"{self.__data.logsData.disposedRebarNumberMaxLoadSec:.2f}",
            "minimumRebarAreaForElementLenght": f"{self.__data.logsData.minimumRebarAreaForElementLenght:.2f}",
            "maxStepTrasv": f"{self.__data.logsData.maxStepTrasv:.2f}",
            "legsNumber": f"{self.__data.logsData.legsNumber:.2f}",
            "legsNumberTrasv": f"{self.__data.logsData.legsNumberTrasv:.2f}",
            "maxStepLongCrit1": f"{self.__data.logsData.maxStepLongCrit1:.2f}",
            "maxStepLongCrit2": f"{self.__data.logsData.maxStepLongCrit2:.2f}",
            "maxStep": f"{self.__data.logsData.maxStep:.2f}",
            "rebarAreaForElementLenght": f"{self.__data.logsData.rebarAreaForElementLenght:.2f}",
        }
        f.add(
            templateName="sheet-PlateMinimumArea-EC211-ita.tex",
            templatePlaceholders=placeHolders,
        )
        self._setFragment(f)
        return f


class BeamMinimumAreaCS(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, data: BeamMinimumArea) -> None:
        super().__init__()
        self.__latexTemplatePath = latexTemplatePath
        self.__data = data

    def setDataModel(self, data: BeamMinimumArea) -> None:
        self.__data = data

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        placeHolders = {
            "elementDescr": f"{self.__data.inputData.elementDescr:s}",
            "keyCode": f"{self.__data.inputData.keyCode:s}",
            "concreteClass": f"{self.__data.inputData.concreteClass:s}",
            "cls_fck": f"{self.__data.logsData.cls_fck:.2f}",
            "cls_fctm": f"{self.__data.logsData.cls_fctm:.2f}",
            "steelClass": f"{self.__data.inputData.steelClass:s}",
            "steel_fyk": f"{self.__data.logsData.steel_fyk:.2f}",
            "hEl": f"{self.__data.inputData.hEl:.2f}",
            "wEl": f"{self.__data.inputData.wEl:.2f}",
            "bt": f"{self.__data.inputData.bt:.2f}",
            "rebarD": f"{self.__data.inputData.rebarD:.2f}",
            "cover": f"{self.__data.inputData.cover:.2f}",
            "bMin": f"{self.__data.inputData.bMin:.2f}",
            "rebarDComp": f"{self.__data.inputData.rebarDComp:.2f}",
            "stirrupD": f"{self.__data.inputData.stirrupD:.2f}",
            "nbLegDirX": f"{self.__data.inputData.nbLegDirX:.2f}",
            "heightUtil": f"{self.__data.logsData.heightUtil:.2f}",
            "areaUtil": f"{self.__data.logsData.areaUtil:.2f}",
            "minimumRebarAreaCrit1": f"{self.__data.logsData.minimumRebarAreaCrit1:.2f}",
            "minimumRebarAreaCrit2": f"{self.__data.logsData.minimumRebarAreaCrit2:.2f}",
            "minimumRebarArea": f"{self.__data.logsData.minimumRebarArea:.2f}",
            "rebarAreaDisposed": f"{self.__data.logsData.rebarAreaDisposed:.2f}",
            "rebarNumber": f"{self.__data.outputData.rebarNumber:.2f}",
            "minimumRebarAreaForElementLenght": f"{self.__data.logsData.minimumRebarAreaForElementLenght:.2f}",
            "minimumLegsForElementLenght": f"{self.__data.logsData.minimumLegsForElementLenght:.2f}",
            "maxStepCrit1": f"{self.__data.logsData.maxStepCrit1:.2f}",
            "maxStepCrit2": f"{self.__data.logsData.maxStepCrit2:.2f}",
            "maxStepCrit3": f"{self.__data.logsData.maxStepCrit3:.2f}",
            "maxStepCrit4": f"{self.__data.logsData.maxStepCrit4:.2f}",
            "stirrupStepMin": f"{self.__data.outputData.stirrupStepMin:.2f}",
            "stirrupCompStepMin": f"{self.__data.outputData.stirrupCompStepMin:.2f}",
        }
        f.add(
            templateName="sheet-BeamMinimumArea-NTC2018-ita.tex",
            templatePlaceholders=placeHolders,
        )
        self._setFragment(f)
        return f



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import math
from typing import Literal, Tuple

from pycivil.EXAUtils.EXAExceptions import EXAExceptions as Ex


def rebarArea(d: float) -> float:
    return d * d * math.pi / 4


def rebarNumber(rebarAreaRequired: float, d: float) -> Tuple[float, float]:
    singleRebarArea = rebarArea(d)
    # nbRebar = math.ceil(As/areaRebar)
    nbRebar = rebarAreaRequired / singleRebarArea

    if rebarAreaRequired == 0.0:
        return 0.0, 0.0

    return nbRebar, nbRebar * singleRebarArea / rebarAreaRequired


def ratioRebarWeight(Vs: float, Vc: float) -> float:
    return Vs * 7850 / Vc


def fcm(fck: float) -> float:
    if fck < 0.0:
        raise Ex("0001", "args fck must be > 0", fck)
    _fcm = fck + 8
    return _fcm


def fctm(fck: float) -> float:
    if fck < 0.0:
        raise Ex("0002", "args fck must be > 0", fck)
    if fck <= 50:
        _fctm = 0.30 * math.pow(fck, 0.666)
    else:
        _fctm = 2.12 * math.log(1 + fcm(fck) / 10)
    return _fctm


def rcSectionBeamAreaMin(
    fck: float, fyk: float, d: float, crit: Literal["c1", "c2", "c12"] = "c12"
) -> float:
    if d < 0.0:
        raise Ex("0003", "args d must be > 0", d)

    Asmin1 = 0.26 * fctm(fck) / fyk * 1000 * d
    Asmin2 = 0.0013 * 1000 * d

    if crit == "c1":
        return Asmin1
    elif crit == "c2":
        return Asmin2
    elif crit == "c12":
        return max(Asmin1, Asmin2)
    else:
        raise Ex("0011", "args crit must be c1, c2 or c12", d)


def rcSectionBeamShearAreaMin(fck: float, fyk: float, b: float) -> float:
    if fck <= 0.0:
        raise Ex("0004", "args b must be > 0", fck)
    if fyk <= 0.0:
        raise Ex("0005", "args b must be > 0", fyk)
    if b <= 0.0:
        raise Ex("0006", "args b must be > 0", b)

    Astmin = 0.08 * b * math.sqrt(fck) / fyk * 1000

    # mm2/m
    return Astmin


def rcSectionPlateShearAreaMin(fck: float, fyk: float) -> float:
    return rcSectionBeamShearAreaMin(fck, fyk, 1000)


def rcSectionBeamStirrupStepsMax(d: float, alpha: float = 90) -> float:
    if d <= 0.0:
        raise Ex("0007", "args d must be > 0", d)

    if not 23 <= alpha <= 90:
        raise Ex("0008", "args alpha must be => 23 and <= 90", alpha)

    smax = 0.75 * d * (1 + math.tan(math.radians(90 - alpha)))

    return smax


def rcSectionPlateStirrupStepsMax(d: float, alpha: float = 90) -> float:
    return rcSectionBeamStirrupStepsMax(d, alpha)


def rcSectionBeamShearLegsMax(d: float) -> float:
    """
    Max trasverse dinstance for legs

    Args:
        d:

    Returns:

    """
    if d <= 0.0:
        raise Ex("0009", "args d must be > 0", d)
    return 0.75 * d


def rcSectionPlateShearLegsMax(d: float) -> float:
    if d <= 0.0:
        raise Ex("0010", "args d must be > 0", d)
    return 1.50 * d


def rcSectionPlateStepMax(
    h: float,
    position: Literal["main", "secondary"] = "main",
    kind: Literal["normal", "maxbending"] = "normal",
) -> float:
    if h < 0.0:
        raise Ex("0008", "args h must be > 0", h)

    if position == "main" and kind == "normal":
        return min(3 * h, 400)
    elif position == "main" and kind == "maxbending":
        return min(2 * h, 250)
    elif position == "secondary" and kind == "normal":
        return min(3.5 * h, 450)
    elif position == "secondary" and kind == "maxbending":
        return min(3 * h, 400)
    else:
        raise Ex("0012", "args position must be 'main' or 'secondary', and kind 'normal' or 'maxbending'", h)




# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import gmsh as gm


def logGmshFile(fileName):
    import sys

    if len(sys.argv) < 2:
        print("Usage: " + sys.argv[0] + " file")
        exit

    # gm.initialize()

    # You can run this tutorial on any file that Gmsh can read, e.g. a mesh file in
    # the MSH format: `python t1.py file.msh'

    # gm.open(sys.argv[1])
    gm.open(fileName)

    # Print the model name and dimension:
    print("Model " + gm.model.getCurrent() + " (" + str(gm.model.getDimension()) + "D)")

    # Geometrical data is made of elementary model `entities', called `points'
    # (entities of dimension 0), `curves' (entities of dimension 1), `surfaces'
    # (entities of dimension 2) and `volumes' (entities of dimension 3). As we have
    # seen in the other Python tutorials, elementary model entities are identified
    # by their dimension and by a `tag': a strictly positive identification
    # number. Model entities can be either CAD entities (from the built-in `geo'
    # kernel or from the OpenCASCADE `occ' kernel) or `discrete' entities (defined
    # by a mesh). `Physical groups' are collections of model entities and are also
    # identified by their dimension and by a tag.

    # Get all the elementary entities in the model, as a vector of (dimension, tag)
    # pairs:
    entities = gm.model.getEntities()

    for e in entities:
        # Dimension and tag of the entity:
        dim = e[0]
        tag = e[1]

        # Mesh data is made of `elements' (points, lines, triangles, ...), defined
        # by an ordered list of their `nodes'. Elements and nodes are identified by
        # `tags' as well (strictly positive identification numbers), and are stored
        # ("classified") in the model entity they discretize. Tags for elements and
        # nodes are globally unique (and not only per dimension, like entities).

        # A model entity of dimension 0 (a geometrical point) will contain a mesh
        # element of type point, as well as a mesh node. A model curve will contain
        # line elements as well as its interior nodes, while its boundary nodes will
        # be stored in the bounding model points. A model surface will contain
        # triangular and/or quadrangular elements and all the nodes not classified
        # on its boundary or on its embedded entities. A model volume will contain
        # tetrahedra, hexahedra, etc. and all the nodes not classified on its
        # boundary or on its embedded entities.

        # Get the mesh nodes for the entity (dim, tag):
        nodeTags, nodeCoords, nodeParams = gm.model.mesh.getNodes(dim, tag)

        # Get the mesh elements for the entity (dim, tag):
        elemTypes, elemTags, elemNodeTags = gm.model.mesh.getElements(dim, tag)

        # Elements can also be obtained by type, by using `getElementTypes()'
        # followed by `getElementsByType()'.

        # Let's print a summary of the information available on the entity and its
        # mesh.

        # * Type and name of the entity:
        type = gm.model.getType(e[0], e[1])
        name = gm.model.getEntityName(e[0], e[1])
        if len(name):
            name += " "
        print("Entity " + name + str(e) + " of type " + type)

        # * Number of mesh nodes and elements:
        numElem = sum(len(i) for i in elemTags)
        print(
            " - Mesh has "
            + str(len(nodeTags))
            + " nodes and "
            + str(numElem)
            + " elements"
        )

        # * Upward and downward adjacencies:
        up, down = gm.model.getAdjacencies(e[0], e[1])
        if len(up):
            print(" - Upward adjacencies: " + str(up))
        if len(down):
            print(" - Downward adjacencies: " + str(down))

        # * Does the entity belong to physical groups?
        physicalTags = gm.model.getPhysicalGroupsForEntity(dim, tag)
        if len(physicalTags):
            s = ""
            for p in physicalTags:
                n = gm.model.getPhysicalName(dim, p)
                if n:
                    n += " "
                s += n + "(" + str(dim) + ", " + str(p) + ") "
            print(" - Physical groups: " + s)

        # * Is the entity a partition entity? If so, what is its parent entity?
        partitions = gm.model.getPartitions(e[0], e[1])
        if len(partitions):
            print(
                " - Partition tags: "
                + str(partitions)
                + " - parent entity "
                + str(gm.model.getParent(e[0], e[1]))
            )

        # * List all types of elements making up the mesh of the entity:
        for t in elemTypes:
            name, dim, order, numv, parv, _ = gm.model.mesh.getElementProperties(t)
            print(
                " - Element type: "
                + name
                + ", order "
                + str(order)
                + " ("
                + str(numv)
                + " nodes in param coord: "
                + str(parv)
                + ")"
            )

    # We can use this to clear all the model data but after i need to reinitialize gm:
    #
    # gm.clear()
    # gm.finalize()



import math
from enum import Enum
from typing import Union, Dict, Literal, Tuple, Any

from adjustText import adjust_text # type: ignore[import-untyped]
from matplotlib import pyplot as plt
from matplotlib.patches import Polygon, Circle

from pycivil.EXAStructural.modeler import SectionModeler
from pycivil.EXAStructural.sections import ConcreteSection
from pycivil.EXAUtils.logging import log


class SectionPlotViewEnum(str, Enum):
    SP_VIEW_SECTION = "SP_VIEW_SECTION"
    SP_VIEW_SECTION_NOID = "SP_VIEW_SECTION_NOID"
    SP_VIEW_NODES = "SP_VIEW_NODES"
    SP_VIEW_SOLID_NODES = "SP_VIEW_SOLID_NODES"
    SP_HIDE_SOLID_NODES = "SP_HIDE_SOLID_NODES"
    SP_HIDE_NODES = "SP_HIDE_NODES"
    SP_HIDE_REBAR_ID = "SP_HIDE_REBAR_ID"
    SP_VIEW_GRID = "SP_VIEW_GRID"
    SP_HIDE_GRID = "SP_HIDE_GRID"
    SP_VIEW_TRIANGLES = "SP_VIEW_TRIANGLES"
    SP_VIEW_TRIANGLE_NOID = "SP_VIEW_TRIANGLE_NOID"
    SP_VIEW_MESH = "SP_VIEW_MESH"
    SP_VIEW_STRESSES = "SP_VIEW_STRESSES"


class SectionPlot:
    def __init__(self):
        self.__fig = plt.figure()
        self.__ax = self.__fig.add_subplot(1, 1, 1)
        self.__ax.set_aspect("equal", "datalim")

        # Data to plot
        self.__rcRect: Union[None, ConcreteSection] = None
        self.__modeler: Union[None, SectionModeler] = None
        self.__vertexStresses: Union[Dict[int, float], None] = None
        self.__rebarStresses: Union[Dict[int, float], None] = None

        # Aspect controls
        self.__colorEdge: str = "black"
        self.__textSize: float = 8
        self.__colorTriaLines: str = "blue"
        self.__colorTriaFill: str = "green"
        self.__colorNodes: str = "red"
        self.__plotShapeEdges: bool = True
        self.__plotTriaLines: bool = False
        self.__plotTriaFill: bool = True
        self.__plotTriaShrinkFill: bool = False
        self.__plotTriaShrink: bool = False
        self.__plotNodes: bool = False
        self.__plotNodesId: bool = False
        self.__plotSolidNodes: bool = True
        self.__plotSolidNodesId: bool = True
        self.__plotStresses: bool = False
        self.__plotRebar: bool = True
        self.__plotMeshTria: bool = False
        self.__plotRebarId: bool = True
        self.__plotGrid: bool = False

        self.__ll: Literal[0, 1, 2] = 0

    @property
    def logLevel(self):
        return self.__ll

    @logLevel.setter
    def logLevel(self, value):
        self.__ll = value

    @property
    def plotRebar(self):
        return self.__plotRebar

    @plotRebar.setter
    def plotRebar(self, value):
        self.__plotRebar = value

    @property
    def colorNodes(self):
        return self.__colorNodes

    @colorNodes.setter
    def colorNodes(self, value):
        self.__colorNodes = value

    @property
    def ax(self):
        return self.__ax

    @ax.setter
    def ax(self, value):
        self.__ax = value

    @property
    def colorTriaFill(self):
        return self.__colorTriaFill

    @colorTriaFill.setter
    def colorTriaFill(self, value):
        self.__colorTriaFill = value

    @property
    def plotTriaShrinkFill(self):
        return self.__plotTriaShrinkFill

    @plotTriaShrinkFill.setter
    def plotTriaShrinkFill(self, value):
        self.__plotTriaShrinkFill = value

    @property
    def textSize(self):
        return self.__textSize

    @textSize.setter
    def textSize(self, value):
        self.__textSize = value

    @property
    def vertexStresses(self) -> Union[Dict[int, float], None]:
        return self.__vertexStresses

    @vertexStresses.setter
    def vertexStresses(self, value: Union[Dict[int, float], None] = None) -> None:
        self.__vertexStresses = value

    @property
    def rebarStresses(self) -> Union[Dict[int, float], None]:
        return self.__rebarStresses

    @rebarStresses.setter
    def rebarStresses(self, value: Union[Dict[int, float], None] = None) -> None:
        self.__rebarStresses = value

    @property
    def plotGrid(self):
        return self.__plotGrid

    @plotGrid.setter
    def plotGrid(self, value):
        self.__plotGrid = value

    @property
    def plotSolidNodesId(self):
        return self.__plotSolidNodesId

    @plotSolidNodesId.setter
    def plotSolidNodesId(self, value):
        self.__plotSolidNodesId = value

    @property
    def modeler(self):
        return self.__modeler

    @modeler.setter
    def modeler(self, value: SectionModeler) -> None:
        self.__modeler = value

    def __plotScaled(self, xDataScaled, yDataScaled, refLenght):
        # Scale from reference lenght in data space
        dx = xDataScaled
        dy = yDataScaled
        headScale = 0.02
        i1 = 0
        i2 = 1
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )
        i1 = 1
        i2 = 2
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )
        i1 = 2
        i2 = 0
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )

    @staticmethod
    def __genTriaData(coords: Tuple[float, float, float, float, float, float]) -> Any:
        xData = [coords[0], coords[2], coords[4], coords[0]]
        yData = [coords[1], coords[3], coords[5], coords[1]]

        xg = (xData[0] + xData[1] + xData[2]) / 3
        yg = (yData[0] + yData[1] + yData[2]) / 3

        xDataScaled = [
            (xData[0] - xg) * 0.8 + xg,
            (xData[1] - xg) * 0.8 + xg,
            (xData[2] - xg) * 0.8 + xg,
        ]

        yDataScaled = [
            (yData[0] - yg) * 0.8 + yg,
            (yData[1] - yg) * 0.8 + yg,
            (yData[2] - yg) * 0.8 + yg,
        ]

        polyData = [
            [xData[0], yData[0]],
            [xData[1], yData[1]],
            [xData[2], yData[2]],
        ]

        polyDataScaled = [
            [xDataScaled[0], yDataScaled[0]],
            [xDataScaled[1], yDataScaled[1]],
            [xDataScaled[2], yDataScaled[2]],
        ]

        return xData, yData, xDataScaled, yDataScaled, polyData, polyDataScaled

    def __plotModeler(self, modeler: SectionModeler) -> None:
        trianglesCoords = modeler.getTrianglesCoords()

        # We need this for operate in data space
        refLenght = math.sqrt(modeler.calcSolidAreaProperties().area)

        ########################################################################
        #                               TRIANGLES                              #
        ########################################################################
        for k in trianglesCoords:
            coords = trianglesCoords[k]
            (
                xData,
                yData,
                xDataScaled,
                yDataScaled,
                polyData,
                polyDataScaled,
            ) = self.__genTriaData(coords)

            ####################################################################
            #                               FILL                               #
            ####################################################################
            if self.__plotTriaFill:
                self.__ax.add_patch(
                    Polygon(polyData, facecolor=self.__colorTriaFill, alpha=0.3)
                )

            ####################################################################
            #                               LINES                              #
            ####################################################################
            if self.__plotTriaLines:
                self.__ax.plot(xData, yData, color=self.__colorTriaLines)

            ####################################################################
            #                       SHRINK LINES AND FILL                      #
            ####################################################################
            if self.__plotTriaShrinkFill:
                self.__ax.add_patch(
                    Polygon(polyDataScaled, facecolor=self.__colorTriaFill, alpha=0.3)
                )

            if self.__plotTriaShrink:
                self.__plotScaled(xDataScaled, yDataScaled, refLenght)

        ########################################################################
        #                               SHAPE                                  #
        ########################################################################
        trianglesIds = modeler.getTrianglesIds()
        if self.__plotShapeEdges:
            segsId = []
            for k in trianglesIds:
                t = trianglesIds[k]
                # Order by id segments
                seg1 = [t[1], t[0]]
                if t[1] > t[0]:
                    seg1 = [t[0], t[1]]
                if seg1 not in segsId:
                    segsId.append(seg1)
                else:
                    segsId.remove(seg1)

                seg2 = [t[2], t[1]]
                if t[2] > t[1]:
                    seg2 = [t[1], t[2]]
                if seg2 not in segsId:
                    segsId.append(seg2)
                else:
                    segsId.remove(seg2)

                seg3 = [t[0], t[2]]
                if t[0] > t[2]:
                    seg3 = [t[2], t[0]]
                if seg3 not in segsId:
                    segsId.append(seg3)
                else:
                    segsId.remove(seg3)

            for ids in segsId:
                self.__ax.plot(
                    [modeler.getNodeX(ids[0]), modeler.getNodeX(ids[1])],
                    [modeler.getNodeY(ids[0]), modeler.getNodeY(ids[1])],
                    color=self.__colorEdge,
                )

        ########################################################################
        #                                NODES                                 #
        ########################################################################
        if self.__plotNodes or self.__plotSolidNodes:
            if self.__plotNodes:
                nodes = modeler.getNodes()
            else:
                nodes = modeler.getSolidNodes()

            for k in nodes:
                node = nodes[k]
                sc = self.__ax.scatter(
                    node.xn,
                    node.yn,
                    marker="o",
                    color=self.__colorNodes,
                    # linewidths = 3.5
                )
                sizeOfAnnotation = math.sqrt(sc.get_sizes()) / 2 + 2

                text_to_put = ""
                if self.__vertexStresses is not None:
                    if self.__plotStresses and self.__plotSolidNodesId:
                        stress = self.__vertexStresses[node.id]
                        text_to_put = f"{stress:.1f}"
                    else:
                        text_to_put = f"{node.id}"

                if self.__plotNodesId or self.__plotSolidNodesId:
                    self.__ax.annotate(
                        text=text_to_put,
                        xy=(node.xn, node.yn),
                        fontsize=self.__textSize,
                        xytext=(sizeOfAnnotation, sizeOfAnnotation),
                        textcoords="offset points",
                        color="red",
                        ha="left",
                        va="center",
                    )

        ########################################################################
        #                                REBAR                                 #
        ########################################################################
        texts = []
        patches = []
        if self.__plotRebar:
            circles = modeler.getCircles()
            for k in circles:
                circle = circles[k]
                x = circle.center.xn
                y = circle.center.yn
                r = circle.radius
                patch = self.__ax.add_patch(Circle((x, y), r))
                self.__ax.add_patch(Circle((x, y), r, fill=False, edgecolor="black"))
                patches.append(patch)

                text_to_put = ""
                if self.__rebarStresses is not None:
                    if self.__plotStresses and self.__plotRebarId:
                        stress = self.__rebarStresses[circle.id]
                        text_to_put = f"{stress:.0f}"
                    else:
                        text_to_put = f"{circle.id}"

                if self.__plotRebarId:
                    # text = self.__ax.annotate(
                    #     text=f"{text_to_put}",
                    #     xy=(x + r, y + r),
                    #     fontsize=self.__textSize,
                    #     # xytext = (sizeOfAnnotation, sizeOfAnnotation),
                    #     # textcoords="offset points",
                    #     ha="left",
                    #     va="center",
                    # )
                    text = self.__ax.text(
                        s=f"{text_to_put}",
                        x=x, # + r,
                        y=y, # + r,
                        fontsize=self.__textSize,
                        # xytext = (sizeOfAnnotation, sizeOfAnnotation),
                        # textcoords="offset points",
                        ha="left",
                        va="center",
                    )
                    texts.append(text)
        #self.__fig.canvas.draw()
        adjust_text(
            texts=texts,
            objects=patches,
            ax=self.__ax,
            max_move=100,
            expand=(1.2, 1.2),
            arrowprops=dict(arrowstyle='->', color='red', linewidth=0.3),
            force_static=(2.0,2.0))

        ########################################################################
        #                               MESHES                                 #
        ########################################################################
        if self.__plotMeshTria:
            for i in range(modeler.meshSize()):
                trianglesCoords = modeler.getTrianglesCoordsAtMesh(i)
                for k in trianglesCoords:
                    coords = trianglesCoords[k]
                    (
                        xData_,
                        yData_,
                        xDataScaled_,
                        yDataScaled_,
                        polyData_,
                        polyDataScaled_,
                    ) = self.__genTriaData(coords)
                    self.__plotScaled(xDataScaled_, yDataScaled_, refLenght)
                    self.__ax.plot(xData_, yData_, color=self.__colorTriaLines)

    def setView(self, opt: SectionPlotViewEnum) -> None:
        if opt == SectionPlotViewEnum.SP_VIEW_MESH:
            self.__plotShapeEdges = False
            self.__plotTriaLines = False
            self.__plotTriaFill = False
            self.__plotTriaShrinkFill = False
            self.__plotTriaShrink = False
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebar = False
            self.__plotMeshTria = True
            self.__plotRebarId = False
            self.__plotGrid = False

        if opt == SectionPlotViewEnum.SP_VIEW_SECTION:
            self.__plotShapeEdges = True
            self.__plotTriaLines = False
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = False
            self.__plotTriaShrink = False
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebar = True
            self.__plotRebarId = True
            self.__plotStresses = False

        if opt == SectionPlotViewEnum.SP_VIEW_SECTION_NOID:
            self.__plotShapeEdges = True
            self.__plotTriaLines = False
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = False
            self.__plotTriaShrink = False
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebar = True
            self.__plotRebarId = False

        if opt == SectionPlotViewEnum.SP_HIDE_REBAR_ID:
            self.__plotRebarId = False

        if opt == SectionPlotViewEnum.SP_VIEW_NODES:
            self.__plotNodes = True
            self.__plotNodesId = True

        if opt == SectionPlotViewEnum.SP_HIDE_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False

        if opt == SectionPlotViewEnum.SP_VIEW_SOLID_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotSolidNodes = True
            self.__plotSolidNodesId = True
            self.__plotStresses = False

        if opt == SectionPlotViewEnum.SP_VIEW_STRESSES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebarId = True
            self.__plotSolidNodes = True
            self.__plotSolidNodesId = True
            self.__plotStresses = True

        if opt == SectionPlotViewEnum.SP_HIDE_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotSolidNodes = False
            self.__plotSolidNodesId = False

        if opt == SectionPlotViewEnum.SP_VIEW_GRID:
            self.__plotGrid = True

        if opt == SectionPlotViewEnum.SP_HIDE_GRID:
            self.__plotGrid = False

        if opt == SectionPlotViewEnum.SP_VIEW_TRIANGLES:
            self.__plotShapeEdges = False
            self.__plotTriaLines = True
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = True
            self.__plotTriaShrink = True
            self.__plotNodes = True
            self.__plotNodesId = True
            self.__plotRebar = False
            self.__plotRebarId = False

        if opt == SectionPlotViewEnum.SP_VIEW_TRIANGLE_NOID:
            self.__plotShapeEdges = False
            self.__plotTriaLines = True
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = True
            self.__plotTriaShrink = True
            self.__plotNodes = True
            self.__plotNodesId = False
            self.__plotRebar = False
            self.__plotRebarId = False

    def plot(self):
        self.__ax.cla()
        #self.__ax.set_facecolor("white")
        if self.__plotGrid:
            self.__ax.grid(True, "both")
            self.__ax.minorticks_on()
            self.__ax.set_facecolor("#EBEBEB")

        if self.__modeler is not None:
            self.__plotModeler(self.__modeler)
        else:
            log("ERR", "Modeler does not assigned !!!", self.__ll)

    @staticmethod
    def show():
        plt.show()

    @staticmethod
    def save(
        fname,
        *,
        transparent=None,
        dpi="figure",
        format=None,
        metadata=None,
        bbox_inches=None,
        pad_inches=0.1,
        facecolor=None,
        edgecolor=None,
        backend=None,
        **kwargs,
    ):
        plt.savefig(
            fname,
            dpi=dpi,
            format=format,
            metadata=metadata,
            bbox_inches=bbox_inches,
            pad_inches=pad_inches,
            facecolor=facecolor,
            edgecolor=edgecolor,
            backend=backend,
            transparent=transparent,
            **kwargs,
        )



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

"""
Created on Sun Nov 07 16:33:00 2021

@author: lpaone
"""

import math
import unittest

import pycivil.EXAStructural.templateRCRect as Est

def build_section(dimW, dimH, Ned, Med, distFromTop, distFromBot, steelAreaBottom, steelAreaTop, homogenization):

    section = Est.RCTemplRectEC2(1, "First Section")

    # Setting dimension concrete
    section.setDimH(dimH)
    section.setDimW(dimW)

    # Setting rebar
    section.addSteelArea("MB", distFromBot, steelAreaBottom)
    section.addSteelArea("MT", distFromTop, steelAreaTop)

    # Setting materials
    section.setMaterials(homogenization=homogenization)

    # Solver with iteration method
    #
    sol = section.solverSLS_NM(N=Ned * 1e3, M=Med * 1e6)

    return sol, section

def buildTest(
    dimW,
    dimH,
    steelAreaTop,
    steelAreaBottom,
    distFromTop,
    distFromBot,
    Ned,
    Med,
    sigmacRef=None,
    sigmasRef=None,
    xiRef=None,
    toll=0.001,
    homogenization=15.0,
    name="",
    sigmacReg=None,
    sigmasReg=None,
    xiReg=None,
    plot=False,
):
    # Solver with iteration method
    #
    sol, section = build_section(
        dimW,
        dimH,
        Ned,
        Med,
        distFromTop,
        distFromBot,
        steelAreaBottom,
        steelAreaTop,
        homogenization
    )
    # From Tuple to List for modification
    sol_list = list(sol)

    # Because solverSLS_NM method use xi measure from top to bottom always, when
    # happen that compressions are on bottom we need to change with top measure.
    if section.getConcrStress()[0] < 0:
        sol_list[2] = dimH - sol_list[2]

    sigma_c_domain, sigma_s_domain, xi_domain, pintersect = (
        section.solverSLS_NM_withDomain(Ned=Ned, Med=Med, points_on_domain=300))

    section.addTensionPoint2d(N=-Ned * 1e3, M=Med * 1e6)
    assert isinstance(pintersect, list)
    section.addTensionPoint2d(N=pintersect[0].x, M=pintersect[0].y)

    tensionPoint = Est.Point2d(-Ned * 1e3, Med * 1e6)
    zeroPoint = Est.Point2d(0, 0)

    lines = [
        [tensionPoint, pintersect[0]],
        [zeroPoint, tensionPoint]
    ]

    if plot:
        section.interactionDomainPlot2d(xLabel="xxx", yLabel="yyy", lines=lines)

    # Reference errors
    #
    if sigmacRef is None:
        sigmacRef = -1
        sigmacErr = -1
    elif sigmacRef == 0.0:
        sigmacErr = abs(sigmacRef - sol_list[0])
    else:
        sigmacErr = abs((sigmacRef - sol_list[0]) / sigmacRef)

    if sigmasRef is None:
        sigmasRef = -1
        sigmasErr = -1
    else:
        sigmasErr = abs((sigmasRef - sol_list[1]) / sigmasRef)

    if xiRef is None:
        xiRef = -1
        xiErr = -1
    else:
        xiErr = abs((xiRef - sol_list[2]) / xiRef)

    if max([sigmacErr, sigmasErr, xiErr]) < toll:
        sentence = "--> OK"
    else:
        sentence = "<<<<<<<<<<<<<<<<<<<<<<<<< KO"

    # Not regression errors
    #
    if sigmacReg is None:
        sigmacErrReg = -1
    elif sigmacReg == 0.0:
        sigmacErrReg = abs(sigmacReg - sol_list[0])
    else:
        sigmacErrReg = abs((sigmacReg - sol_list[0]) / sigmacReg)

    if sigmasReg is None:
        sigmasReg = -1
        sigmasErrReg = -1
    else:
        sigmasErrReg = abs((sigmasReg - sol_list[1]) / sigmasReg)

    if xiReg is None:
        xiErrReg = -1
    else:
        xiErrReg = abs((xiReg - sol_list[2]) / xiReg)

    if max([sigmacErrReg, sigmasErrReg, xiErrReg]) < toll:
        sentenceReg = "--> OK"
    else:
        sentenceReg = "<<<<<<<<<<<<<<<<<<<<<<<<< KO"

    # From elastic domain errors
    #
    if sigma_c_domain != 0.0:
        sigmacErrDom = abs((sigma_c_domain - sol_list[0]) / sigma_c_domain)
    else:
        sigmacErrDom = abs(sigma_c_domain - sol_list[0])

    sigmasErrDom = abs((sigma_s_domain - sol_list[1]) / sigma_s_domain)
    xiErrDom = abs((xi_domain - sol_list[2]) / xi_domain)

    if max([sigmacErrDom, sigmasErrDom, xiErrDom]) < toll:
        sentenceDom = "--> OK"
    else:
        sentenceDom = "<<<<<<<<<<<<<<<<<<<<<<<<< KO"

    print(
        "--------------------------------------------------------------------------------"
    )
    print(f"Ned = {Ned:3.0f} Med = {Med:3.0f}")
    print("Concrete stresses: ", section.getConcrStress())
    print("Steel    stresses: ", section.getSteelStress())
    print(
        "W = %3.f H = %3.f area Top = %3.f area Bottom = %3.f topDist = %3.f botDist = %3.f "
        % (dimW, dimH, steelAreaTop, steelAreaBottom, distFromTop, distFromBot)
    )
    print(
        "      ref sigmac =%8.3f -       ref sigmas =%8.3f -       ref xi =%8.3f name of reference ** %s **"
        % (sigmacRef, sigmasRef, xiRef, name)
    )
    assert sigmacReg is not None
    print(
        f"      reg sigmac =%8.3f -       reg sigmas =%8.3f -       reg xi =%8.3f"
        % (sigmacReg, sigmasReg, xiReg)
    )
    assert sol_list[0] is not None
    assert sol_list[1] is not None
    assert sol_list[2] is not None
    print(
        "          sigmac =%8.3f -           sigmas =%8.3f -           xi =%8.3f from classical algorythm"
        % (sol_list[0], sol_list[1], sol_list[2])
    )
    print(
        "      dom sigmac =%8.3f -       dom sigmas =%8.3f -       dom xi =%8.3f from elastic domain"
        % (sigma_c_domain, sigma_s_domain, xi_domain)
    )
    print(
        "--------------------------------------------------------------------------------"
    )
    print(
        "ERR   ref sigmac =%8.3f -       ref sigmas =%8.3f -       ref xi =%8.3f - RESULT: %s"
        % (sigmacErr, sigmasErr, xiErr, sentence)
    )
    print(
        "ERR   reg sigmac =%8.3f -       reg sigmas =%8.3f -       reg xi =%8.3f - RESULT: %s"
        % (sigmacErrReg, sigmasErrReg, xiErrReg, sentenceReg)
    )
    print(
        "ERR   dom sigmac =%8.3f -       dom sigmas =%8.3f -       dom xi =%8.3f - RESULT: %s"
        % (sigmacErrDom, sigmasErrDom, xiErrDom, sentenceDom)
    )
    print(
        "--------------------------------------------------------------------------------"
    )
    return sigmacErr, sigmasErr, xiErr, sentence, sentenceReg, sentenceDom


class Test(unittest.TestCase):
    def test_101_sws_excel_1_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1300.0,
            steelAreaTop=1571.0,
            steelAreaBottom=3142.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-964.0,
            Med=+1081.0,
            sigmacRef=-6.17,
            sigmasRef=168.70,
            toll=0.005,
            name="excel 1 (+)",
            sigmacReg=-6.166,
            sigmasReg=168.628,
            xiReg=442.749,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_102_sws_excel_2_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2300.0,
            steelAreaTop=1901.0,
            steelAreaBottom=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-2123.6,
            Med=+941.5,
            sigmacRef=-1.90,
            sigmasRef=0.94,
            toll=0.035,
            name="excel 2 (+)",
            sigmacReg=-1.904,
            sigmasReg=0.940,
            xiReg=2178.314,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_103_sws_excel_3_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2200.0,
            steelAreaTop=1901.0,
            steelAreaBottom=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-2123.8,
            Med=+1695.4,
            sigmacRef=-3.39,
            sigmasRef=34.78,
            toll=0.005,
            name="excel 3 (+)",
            sigmacReg=-3.389,
            sigmasReg=34.788,
            xiReg=1276.502,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_104_sws_excel_4_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2200.0,
            steelAreaTop=1901.0,
            steelAreaBottom=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-1920.6,
            Med=+1940.8,
            sigmacRef=-4.02,
            sigmasRef=65.97,
            toll=0.005,
            name="excel 4 (+)",
            sigmacReg=-4.019,
            sigmasReg=65.983,
            xiReg=1026.430,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_105_sws_excel_5_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1700.0,
            steelAreaTop=1901.0,
            steelAreaBottom=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-1639.2,
            Med=+1940.8,
            sigmacRef=-6.55,
            sigmasRef=158.13,
            toll=0.005,
            name="excel 5 (+)",
            sigmacReg=-6.546,
            sigmasReg=158.145,
            xiReg=632.011,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_106_sws_excel_6_plus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1000.0,
            steelAreaTop=1571.0,
            steelAreaBottom=1571.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-678.5,
            Med=+574.6,
            sigmacRef=-6.30,
            sigmasRef=219.37,
            toll=0.005,
            name="excel 6 (+)",
            sigmacReg=-6.298,
            sigmasReg=219.324,
            xiReg=286.000,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_101_sws_excel_1_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1300.0,
            steelAreaBottom=1571.0,
            steelAreaTop=3142.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-964.0,
            Med=-1081.0,
            sigmacRef=-6.17,
            sigmasRef=168.70,
            toll=0.005,
            name="excel 1 (-)",
            sigmacReg=-6.166,
            sigmasReg=168.628,
            xiReg=442.749,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_102_sws_excel_2_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2300.0,
            steelAreaBottom=1901.0,
            steelAreaTop=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-2123.6,
            Med=-941.5,
            sigmacRef=-1.90,
            sigmasRef=0.94,
            toll=0.040,
            name="excel 2 (-)",
            sigmacReg=-1.904,
            sigmasReg=0.940,
            xiReg=2178.314,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_103_sws_excel_3_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2200.0,
            steelAreaBottom=1901.0,
            steelAreaTop=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-2123.8,
            Med=-1695.4,
            sigmacRef=-3.39,
            sigmasRef=34.78,
            toll=0.005,
            name="excel 3 (-)",
            sigmacReg=-3.389,
            sigmasReg=34.788,
            xiReg=1276.502,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_104_sws_excel_4_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=2200.0,
            steelAreaBottom=1901.0,
            steelAreaTop=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-1920.6,
            Med=-1940.8,
            sigmacRef=-4.02,
            sigmasRef=65.97,
            toll=0.005,
            name="excel 4 (-)",
            sigmacReg=-4.019,
            sigmasReg=65.983,
            xiReg=1026.430,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_105_sws_excel_5_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1700.0,
            steelAreaBottom=1901.0,
            steelAreaTop=3801.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-1639.2,
            Med=-1940.8,
            sigmacRef=-6.55,
            sigmasRef=158.13,
            toll=0.005,
            name="excel 5 (-)",
            sigmacReg=-6.546,
            sigmasReg=158.145,
            xiReg=632.011,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_106_sws_excel_6_minus(self):
        res = buildTest(
            dimW=1000.0,
            dimH=1000.0,
            steelAreaBottom=1571.0,
            steelAreaTop=1571.0,
            distFromTop=50.0,
            distFromBot=50.0,
            Ned=-678.5,
            Med=-574.6,
            sigmacRef=-6.30,
            sigmasRef=219.37,
            toll=0.005,
            name="excel 6 (-)",
            sigmacReg=-6.298,
            sigmasReg=219.324,
            xiReg=286.000,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_107_exagon_5_plus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=1256.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=-0.0,
            Med=+1000.0,
            sigmacRef=-43.22,
            sigmasRef=1485.70,
            toll=0.005,
            name="EXAGONE test 5 (+)",
            sigmacReg=-43.222,
            sigmasReg=1484.733,
            xiReg=176.286,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_107_exagon_5_minus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=1256.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=-0.0,
            Med=-1000.0,
            sigmacRef=-43.22,
            sigmasRef=1485.70,
            toll=0.005,
            name="EXAGONE test 5 (-)",
            sigmacReg=-43.222,
            sigmasReg=1484.733,
            xiReg=176.286,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_108_prefle_1_plus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=312.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=-100.0,
            Med=+100.0,
            sigmacRef=-6.94573,
            sigmasRef=184.056,
            xiRef=209.6427517,
            toll=0.340,
            name="PREFLE+ (+)",
            sigmacReg=-6.118,
            sigmasReg=122.680,
            xiReg=248.196,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_108_prefle_1_minus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=312.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=-100.0,
            Med=-100.0,
            sigmacRef=-8.02624,
            sigmasRef=428.737,
            xiRef=127.1615167,
            toll=0.340,
            name="PREFLE+",
            sigmacReg=-7.063,
            sigmasReg=425.114,
            xiReg=115.704,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_109_manual_1_plus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=1256.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=+1000.0,
            Med=0.0,
            sigmacRef=0.0,
            sigmasRef=398.089,
            toll=0.005,
            name="Manual reference (+)",
            sigmacReg=0.0,
            sigmasReg=398.0891,
            xiReg=math.inf,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")

    def test_109_manual_1_minus(self):
        res = buildTest(
            dimW=300.0,
            dimH=600.0,
            steelAreaBottom=1256.0,
            steelAreaTop=1256.0,
            distFromTop=20.0,
            distFromBot=20.0,
            Ned=-1000.0,
            Med=0.0,
            sigmacRef=-4.59,
            sigmasRef=-68.90,
            toll=0.005,
            name="Manual reference (-)",
            sigmacReg=-4.5938,
            sigmasReg=-68.9084,
            xiReg=math.inf,
        )
        sentence = res[3]
        sentenceReg = res[4]
        sentenceDom = res[5]
        self.assertEqual(sentence, "--> OK")
        self.assertEqual(sentenceReg, "--> OK")
        self.assertEqual(sentenceDom, "--> OK")


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

"""
Created on Sun Nov 19 11:02:00 2023

@author: lpaone
"""
import math
import unittest
from typing import List

from pycivil.EXAGeometry.clouds import PointCloud2d
from pycivil.EXAGeometry.geometry import Point2d, Seg2d


class Test(unittest.TestCase):
    def test_001_constructor(self):
        PointCloud2d([[1, 2], [2, 3]])

        self.assertRaises(ValueError, PointCloud2d, [2, 3])

    def test_002_inspect(self):
        pc = PointCloud2d([[1, 2], [2, 3]])
        self.assertEqual(pc[0], Point2d(1, 2))
        self.assertEqual(pc[1], Point2d(2, 3))
        self.assertRaises(IndexError, pc.__getitem__, 2)

        for i, e in enumerate(pc):
            print(f"At {i} item is {e}")
            self.assertEqual(e, pc[i])

        self.assertEqual(len(pc), 2)

        print(pc.getPoints())

    def test_003_intersect(self):
        p1 = Point2d(-1.0, -1.0)
        p2 = Point2d(+1.0, -1.0)
        p3 = Point2d(+1.0, +1.0)
        p4 = Point2d(-1.0, +1.0)
        seg1 = Seg2d(p1, p2)
        seg2 = Seg2d(p2, p3)
        seg3 = Seg2d(p3, p4)
        seg4 = Seg2d(p4, p1)

        self.assertTrue(seg1.p0 == p1 and seg1.p1 == p2)
        self.assertTrue(seg2.p0 == p2 and seg2.p1 == p3)
        self.assertTrue(seg3.p0 == p3 and seg3.p1 == p4)
        self.assertTrue(seg4.p0 == p4 and seg4.p1 == p1)

        pint = seg1.intersect(seg2)
        self.assertTrue(pint == p2)
        self.assertFalse(pint.isNull())

        pint = seg2.intersect(seg3)
        self.assertFalse(pint.isNull())
        self.assertTrue(pint == p3)

        pint = seg3.intersect(seg4)
        self.assertFalse(pint.isNull())
        self.assertTrue(pint == p4)

        pint = seg1.intersect(seg3)
        self.assertTrue(pint.isNull())

        pint = seg2.intersect(seg4)
        self.assertTrue(pint.isNull())

        pint = seg1.intersect(Seg2d(p2, p4))
        self.assertFalse(pint.isNull())
        self.assertTrue(pint == p2)

        pint = seg1.intersect(Seg2d(p1, p3))
        self.assertFalse(pint.isNull())
        self.assertTrue(pint == p1)

    def test_004_convexity(self):
        nbAngle = 90
        ray = 1.0
        points = []
        for i in range(nbAngle):
            p = Point2d(
                ray * math.cos(math.pi / nbAngle * i),
                ray * math.sin(math.pi / nbAngle * i),
            )
            points.append(p)
        pc = PointCloud2d(points)
        self.assertTrue(pc.isConvex())

    def test_005_contained_false(self):
        nbAngle = 4
        ray = 1.0
        points: List[Point2d] = []
        for i in range(nbAngle):
            p = Point2d(
                ray * math.cos(2 * math.pi / nbAngle * i),
                ray * math.sin(2 * math.pi / nbAngle * i),
            )
            points.append(p)
        pc = PointCloud2d(points)

        self.assertTrue(pc.isConvex())

        bb = pc.boundingBox()
        ro = (bb[1] - bb[0], bb[3] - bb[2])

        res = pc.contains(1.0, 1.0, ro=ro)
        contained = res[0]
        val = res[1]
        if isinstance(val, list):
            points = val
        factor = res[2]
        indexes = res[3]
        assert isinstance(points, list)
        assert isinstance(factor, float)
        assert isinstance(indexes, tuple)

        self.assertEqual(contained, False)
        self.assertEqual(len(points), 2)
        p0 = Point2d(0.5, 0.5)
        p1 = Point2d(-0.5, -0.5)
        self.assertAlmostEqual(points[0].x, p0.x, delta=1e-16)
        self.assertAlmostEqual(points[0].y, p0.y, delta=1e-16)
        self.assertAlmostEqual(points[1].x, p1.x, delta=1e-16)
        self.assertAlmostEqual(points[1].y, p1.y, delta=1e-16)

    def test_006_contained_true(self):
        nbAngle = 4
        ray = 1.0
        points = []
        for i in range(nbAngle):
            p = Point2d(
                ray * math.cos(2 * math.pi / nbAngle * i),
                ray * math.sin(2 * math.pi / nbAngle * i),
            )
            points.append(p)
        pc = PointCloud2d(points)

        self.assertTrue(pc.isConvex())

        bb = pc.boundingBox()
        ro = (bb[1] - bb[0], bb[3] - bb[2])

        res = pc.contains(0.25, 0.25, ro=ro)

        contained = res[0]
        val = res[1]
        if isinstance(val, list):
            points = val
        factor = res[2]
        indexes = res[3]
        assert isinstance(points, list)
        assert isinstance(factor, float)
        assert isinstance(indexes, tuple)

        self.assertEqual(contained, True)
        self.assertEqual(len(points), 2)
        p0 = Point2d(0.5, 0.5)
        p1 = Point2d(-0.5, -0.5)
        self.assertAlmostEqual(points[0].x, p0.x, delta=1e-16)
        self.assertAlmostEqual(points[0].y, p0.y, delta=1e-16)
        self.assertAlmostEqual(points[1].x, p1.x, delta=1e-16)
        self.assertAlmostEqual(points[1].y, p1.y, delta=1e-16)

    def test_007_contained_horizontal(self):
        nbAngle = 4
        ray = 1.0
        points = []
        for i in range(nbAngle):
            p = Point2d(
                ray * math.cos(2 * math.pi / nbAngle * i),
                ray * math.sin(2 * math.pi / nbAngle * i),
            )
            points.append(p)
        pc = PointCloud2d(points)

        self.assertTrue(pc.isConvex())

        bb = pc.boundingBox()
        ro = (bb[1] - bb[0], bb[3] - bb[2])

        res = pc.contains(0.25, 0.0, ro=ro)
        contained = res[0]
        val = res[1]
        if isinstance(val, list):
            points = val
        factor = res[2]
        indexes = res[3]
        assert isinstance(points, list)
        assert isinstance(factor, float)
        assert isinstance(indexes, tuple)

        self.assertEqual(contained, True)
        self.assertEqual(len(points), 2)
        p0 = Point2d(+1.0, 0.0)
        p1 = Point2d(-1.0, 0.0)
        self.assertAlmostEqual(points[0].x, p0.x, delta=1e-15)
        self.assertAlmostEqual(points[0].y, p0.y, delta=1e-15)
        self.assertAlmostEqual(points[1].x, p1.x, delta=1e-15)
        self.assertAlmostEqual(points[1].y, p1.y, delta=1e-15)

    def test_008_contained_vertical(self):
        nbAngle = 4
        ray = 1.0
        points = []
        for i in range(nbAngle):
            p = Point2d(
                ray * math.cos(2 * math.pi / nbAngle * i),
                ray * math.sin(2 * math.pi / nbAngle * i),
            )
            points.append(p)
        pc = PointCloud2d(points)

        self.assertTrue(pc.isConvex())

        bb = pc.boundingBox()
        ro = (bb[1] - bb[0], bb[3] - bb[2])

        res = pc.contains(0.0, 0.25, ro=ro)
        contained = res[0]
        val = res[1]
        if isinstance(val, list):
            points = val
        factor = res[2]
        indexes = res[3]
        assert isinstance(points, list)
        assert isinstance(factor, float)
        assert isinstance(indexes, tuple)

        self.assertEqual(contained, True)
        self.assertEqual(len(points), 2)
        p0 = Point2d(0.0, +1.0)
        p1 = Point2d(0.0, -1.0)
        self.assertAlmostEqual(points[0].x, p0.x, delta=1e-15)
        self.assertAlmostEqual(points[0].y, p0.y, delta=1e-15)
        self.assertAlmostEqual(points[1].x, p1.x, delta=1e-15)
        self.assertAlmostEqual(points[1].y, p1.y, delta=1e-15)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import copy
import math
from typing import List, Tuple, Union, TypedDict, Unpack

import numpy as np

import matplotlib
matplotlib.use('Agg')

import matplotlib.pyplot as plt
from matplotlib.axes import Axes
from matplotlib.patches import PathPatch
from matplotlib.path import Path
from matplotlib.ticker import FormatStrFormatter

from pycivil.EXAGeometry.clouds import PointCloud2d
from pycivil.EXAGeometry.geometry import Point2d
from adjustText import adjust_text # type: ignore[import-untyped]


class Geometry2DPlot:
    def __init__(
        self,
        nrows: int = 1,
        ncols: int = 1,
        figsize: Tuple[float, float] = (7, 7),
        sx: float = 1,
        sy: float = 1,
    ):
        self.__fig = plt.figure(figsize=figsize)

        # Adjust layout
        # self.__fig.subplots_adjust(hspace=0.9, wspace=0.9)

        self.__ax: List[Axes] = [
            self.__fig.add_subplot(nrows, ncols, i + 1) for i in range(nrows * ncols)
        ]
        self.__titles: List[str] = [""] * len(self.__ax)
        self.__xlabels: List[str] = [""] * len(self.__ax)
        self.__ylabels: List[str] = [""] * len(self.__ax)

        # Data to plot
        self.__points: List[Point2d] = []
        self.__pointArray: List[List[Point2d]] = []
        self.__pointClouds: List[PointCloud2d] = []

        # Scale Factor
        self.__sx = sx
        self.__sy = sy

        # Aspect controls
        self.__pointSize: float = 30
        self.__textSize: float = 12
        self.__showPointIndex: bool = True
        self.__showPointIndexArray: bool = True
        self.__showLineArray: bool = False
        self.__showArrow: bool = True
        self.__showLine: bool = True
        self.__colorLines: str = "black"
        self.__colorPoints: str = "red"
        self.__colorPointsArray: List[List[str]] = []

    @property
    def pointSize(self):
        return self.__pointSize

    @pointSize.setter
    def pointSize(self, value: float) -> None:
        self.__pointSize = value

    @property
    def textSize(self):
        return self.__textSize

    @textSize.setter
    def textSize(self, value: float) -> None:
        self.__textSize = value

    def setXLabel(self, xlabel: List[str]) -> None:
        self.__xlabels = xlabel

    def setYLabel(self, ylabel: List[str]) -> None:
        self.__ylabels = ylabel

    def setTitles(self, titles: List[str]) -> None:
        self.__titles = titles

    def showItems(
        self,
        pointIndex: bool = True,
        arrow: bool = True,
        line: bool = True,
        lineArray: bool = False,
        pointIndexArray: bool = True,
    ) -> None:
        self.__showPointIndex = pointIndex
        self.__showArrow = arrow
        self.__showLine = line
        self.__showLineArray = lineArray
        self.__showPointIndexArray = pointIndexArray

    def addPoint(self, point: Point2d) -> None:
        self.__points.append(point)

    def addPointArray(self, array: List[Point2d], colorArray: List[str] | None = None) -> None:
        if colorArray is None:
            colorArray = []
        self.__pointArray.append(array)

        if len(colorArray) == 0:
            self.__colorPointsArray.append(["black"] * len(array))
            return

        if len(colorArray) == 1:
            self.__colorPointsArray.append([colorArray[0] * len(array)])
            return

        if len(colorArray) not in [0, 1, len(array)]:
            raise ValueError(
                f"Len of array {len(array)} must be equal to \
                            len of colorArray {len(colorArray)}"
            )
        else:
            self.__colorPointsArray.append(colorArray)
            return

    def addPointCloud(self, cloud: PointCloud2d) -> None:
        self.__pointClouds.append(cloud)

    def __plotFromPointArray(self, points: List[List[Point2d]]) -> None:
        if len(self.__ax) != len(points):
            print("ERR: Can't plot many set of points with one axes")

        else:
            for idx, pts in enumerate(points):
                # save options
                showLineOld = self.__showLine
                self.__showLine = self.__showLineArray

                showPointIndexOld = self.__showPointIndex
                self.__showPointIndex = self.__showPointIndexArray

                self.__plotFromPoint2dLst(
                    pts, self.__ax[idx], self.__colorPointsArray[idx]
                )

                # restore options
                self.__showLine = showLineOld
                self.__showPointIndex = showPointIndexOld

    def __plotFromPoint2dLst(
        self, points: List[Point2d], axes: Axes, color: Union[str, List[str]]
    ) -> None:
        for i, p in enumerate(points):
            if isinstance(color, str):
                sc = axes.scatter(
                    [p.x * self.__sx],
                    [p.y * self.__sy],
                    color=color,
                    s=self.__pointSize,
                )
            else:
                sc = axes.scatter(
                    [p.x * self.__sx],
                    [p.y * self.__sy],
                    color=color[i],
                    s=self.__pointSize,
                )

            if self.__showPointIndex:
                sizeOfAnnotation = math.sqrt(sc.get_sizes()[0]) / 2 + 2
                axes.annotate(
                    text=f"{i}",
                    xy=(p.x * self.__sx, p.y * self.__sy),
                    xytext=(sizeOfAnnotation, 0),
                    fontsize=self.__textSize,
                    textcoords="offset points",
                    ha="left",
                    va="center",
                )

            if i < len(points) - 1:
                p0 = points[i]
                p1 = points[i + 1]

                if self.__showLine:
                    axes.plot(
                        [p0.x * self.__sx, p1.x * self.__sx],
                        [p0.y * self.__sy, p1.y * self.__sy],
                        color=self.__colorLines,
                    )

                if self.__showArrow:
                    centerOfArrow = [
                        (p0.x * self.__sx + p1.x * self.__sx) / 2,
                        (p0.y * self.__sy + p1.y * self.__sy) / 2,
                    ]
                    direction = [
                        (p1.x * self.__sx - p0.x * self.__sx),
                        (p1.y * self.__sx - p0.y),
                    ]
                    lenght = math.sqrt(
                        math.pow(direction[0], 2) + math.pow(direction[1], 2)
                    )
                    if not lenght == 0:
                        versor = [
                            direction[0] / lenght / 100,
                            direction[1] / lenght / 100,
                        ]
                        axes.annotate(
                            "",
                            xy=(centerOfArrow[0], centerOfArrow[1]),
                            xytext=(
                                centerOfArrow[0] + versor[0],
                                centerOfArrow[1] + versor[1],
                            ),
                            fontsize=self.__textSize * 2,
                            arrowprops={
                                "arrowstyle": "<-",
                            },
                        )
                    else:
                        print(
                            f"Point at coords {p0} duplicate with index {i} and {i + 1}"
                        )

    def __plotFromPointCloudLst(self, points: List[PointCloud2d]) -> None:

        if len(self.__titles) == len(self.__ax):
            for i, t in enumerate(self.__titles):
                self.__ax[i].set_title(t)
        else:
            if len(self.__titles) == 1:
                for a in self.__ax:
                    a.set_title(self.__titles[0])
            else:
                print("ERR: Can't set titles because lenght !!!")

        if len(self.__xlabels) == len(self.__ax):
            for i, t in enumerate(self.__xlabels):
                self.__ax[i].set_xlabel(t)
        else:
            if len(self.__xlabels) == 1:
                for a in self.__ax:
                    a.set_xlabel(self.__xlabels[0])
            else:
                print("ERR: Can't set xlabels because lenght !!!")

        if len(self.__ylabels) == len(self.__ax):
            for i, t in enumerate(self.__ylabels):
                self.__ax[i].set_ylabel(t)
        else:
            if len(self.__ylabels) == 1:
                for a in self.__ax:
                    a.set_ylabel(self.__ylabels[0])
            else:
                print("ERR: Can't set ylabels because lenght !!!")

        if len(self.__ax) == 1:
            for p in points:
                self.__plotFromPoint2dLst(
                    p.getPoints(), self.__ax[0], self.__colorPoints
                )

        else:
            if len(self.__ax) != len(points):
                print("ERR: Can't plot many set of points with one axes")

            else:
                for idx, p in enumerate(points):
                    self.__plotFromPoint2dLst(
                        p.getPoints(), self.__ax[idx], self.__colorPoints
                    )

    def plot(self):

        # Points
        self.__plotFromPoint2dLst(self.__points, self.__ax[0], self.__colorPoints)

        # Point cloud
        self.__plotFromPointCloudLst(self.__pointClouds)

        # Point array
        if len(self.__pointArray) > 0:
            self.__plotFromPointArray(self.__pointArray)

        self.__fig.tight_layout(pad=2)

    @staticmethod
    def show():
        plt.show()

    @staticmethod
    def save(
        fname,
        *,
        transparent=None,
        dpi="figure",
        format=None,
        metadata=None,
        bbox_inches=None,
        pad_inches=0.1,
        facecolor="auto",
        edgecolor="auto",
        backend=None,
        **kwargs,
    ):

        plt.savefig(
            fname,
            dpi=dpi,
            format=format,
            metadata=metadata,
            bbox_inches=bbox_inches,
            pad_inches=pad_inches,
            facecolor=facecolor,
            edgecolor=edgecolor,
            backend=backend,
            transparent=transparent,
            **kwargs,
        )

class KvargsPlot(TypedDict):
    export: str | None
    dpi: int | None
    xLabel: str
    yLabel: str
    titleAddStr: str
    tensionPoints: List[List[float]]
    scale_Nx: float
    scale_Mz: float
    lines: List[List[Point2d]]
    markers: bool
    savingSingleDomains: bool
    hotPoints: bool | List[bool]
    printDomains: List[int] | None

def interactionDomainBasePlot2d(
        vertices: List[List[float]],
        fields: List[float],
        **kwargs: Unpack[KvargsPlot]
) -> None:
    if "export" in kwargs:
        fileName = kwargs["export"]
    else:
        fileName = None

    if "dpi" in kwargs:
        dpi = kwargs["dpi"]
    else:
        dpi = None

    if "xLabel" in kwargs:
        xLabel = kwargs["xLabel"]
    else:
        xLabel = "Nz [N]"

    if "yLabel" in kwargs:
        yLabel = kwargs["yLabel"]
    else:
        yLabel = "Mz [Nmm]"

    if "titleAddStr" in kwargs:
        titleAddStr = kwargs["titleAddStr"]
    else:
        titleAddStr = None

    if "tensionPoints" in kwargs:
        tensionPoints = kwargs["tensionPoints"]
    else:
        tensionPoints = None

    if "scale_Nx" in kwargs:
        scale_Nx = kwargs["scale_Nx"]
    else:
        scale_Nx = 0.001

    if "scale_Mz" in kwargs:
        scale_Mz = kwargs["scale_Mz"]
    else:
        scale_Mz = 0.000001

    if "lines" in kwargs:
        lines = kwargs["lines"]
    else:
        lines = []

    if "markers" in kwargs:
        markers = kwargs["markers"]
    else:
        markers = True

    if "savingSingleDomains" in kwargs:
        savingSingleDomains = kwargs["savingSingleDomains"]
    else:
        savingSingleDomains = False

    if "hotPoints" in kwargs:
        hotPoints = kwargs["hotPoints"]
    else:
        hotPoints = False

    if "printDomains" in kwargs:
        printDomains = kwargs["printDomains"]
        if printDomains is None:
            printDomains = list(range(0, len(vertices)))
    else:
        printDomains = list(range(0, len(vertices)))

    """
    if "pointsScale" in kwargs:
        pointsScale = kwargs["pointsScale"]
    else:
        pointsScale = 40
    """
    if isinstance(vertices, list) and len(vertices) != 0:
        # Only one diagrammpython
        if isinstance(vertices[0][0], float) and isinstance(vertices[0][1], float):
            verticesList = [vertices]
            fieldsList = [fields]
        else:
            verticesList = vertices
            fieldsList = fields
    else:
        raise ValueError(
            "Vertices must be a list with lenght > 0 !!! Maybe you need build domain."
        )

    fig, ax = plt.subplots()

    if tensionPoints is not None:
        if len(tensionPoints) > 0:
            tp = np.array(tensionPoints, float)
            ax.scatter(
                tp[:, 0] * scale_Nx,
                tp[:, 1] * scale_Mz,
                marker="+",
                s=2000,
                color=[0, 0, 0],
            )

    # plt.autoscale(enable = None)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.xticks(rotation="vertical")
    plt.margins(0.0)
    plt.subplots_adjust(bottom=0.25)
    plt.subplots_adjust(left=0.20)

    if len(lines) != 0:
        for ll in lines:
            x1 = ll[0].x * scale_Nx
            y1 = ll[0].y * scale_Mz
            x2 = ll[1].x * scale_Nx
            y2 = ll[1].y * scale_Mz
            plt.plot([x1, x2], [y1, y2], "k-", lw=2)

    strTitle = "Interaction Domain Nx - Mz"
    if titleAddStr is not None:
        plt.subplots_adjust(top=0.87)
        strTitle = strTitle + "\n" + titleAddStr

    ax.set_title(strTitle)
    ax.grid(True)
    ax.yaxis.set_major_formatter(FormatStrFormatter("%.2e"))
    ax.xaxis.set_major_formatter(FormatStrFormatter("%.2e"))

    # Cloning list only for plot only with deep
    verticesListCopy = copy.deepcopy(verticesList)

    # Removing columns from 2 to n that contains others values
    for _idx, vertices_i in enumerate(verticesListCopy):
        for idy, v in enumerate(vertices_i):
            vertices_i[idy] = [v[0], v[1]]

    for idx, vertices_i in enumerate(verticesListCopy):

        if idx not in printDomains:
            continue

        codes = [Path.MOVETO] + [Path.LINETO] * (len(vertices_i) - 1)

        vertices_i_for_plot = np.array(vertices_i, float)

        # setting scale
        vertices_i_for_plot[:, 0] = vertices_i_for_plot[:, 0] * scale_Nx
        vertices_i_for_plot[:, 1] = vertices_i_for_plot[:, 1] * scale_Mz

        # setting color for hot and cold section
        _edgecolor = "black"
        if isinstance(hotPoints, bool):
            if not hotPoints:
                _edgecolor = "black"
        if isinstance(hotPoints, list):
            if not hotPoints[idx]:
                _edgecolor = "black"
            else:
                _edgecolor = "red"

        path = Path(vertices_i_for_plot, codes)
        pathpatch = PathPatch(path, facecolor="None", edgecolor=_edgecolor)

        ax.add_patch(pathpatch)

        if markers:
            vertex = path.vertices
            assert isinstance(vertex,np.ndarray)
            x = vertex[:, 0]
            y = vertex[:, 1]
            colorsMap = np.array(
                [
                    [1, 0, 0, 0.5],
                    [0, 1, 0, 0.5],
                    [0, 0, 1, 0.5],
                    [1, 0, 1, 0.5],
                    [1, 0, 0, 1.0],
                    [0, 1, 0, 1.0],
                    [0, 0, 1, 1.0],
                    [1, 0, 1, 1.0],
                ]
            )

            # Displayng Domain Different colors
            scatt_colors = []
            for f in fieldsList[idx]:
                if f == 1.0:
                    scatt_colors.append(0)
                elif f == 2.0:
                    scatt_colors.append(1)
                elif f == 3.0:
                    scatt_colors.append(2)
                elif f == 4.0:
                    scatt_colors.append(3)
                elif f == 11.0:
                    scatt_colors.append(4)
                elif f == 12.0:
                    scatt_colors.append(5)
                elif f == 13.0:
                    scatt_colors.append(6)
                elif f == 14.0:
                    scatt_colors.append(7)
                else:
                    raise ValueError("The value of f is <%1.4f> not mapped!!!" % f)

            ax.scatter(x, y, marker="o", s=40, c=colorsMap[scatt_colors])

        if savingSingleDomains:
            ax.autoscale_view()
            if titleAddStr is not None:
                plt.savefig(titleAddStr + "_domain_" + str("%1.0i" % idx) + ".png")

    ax.autoscale_view()

    if fileName is not None:
        if dpi is not None:
            print("save figure to file name -->", fileName)
            plt.style.use("classic")
            plt.savefig(fileName, bbox_inches="tight", dpi=600)
        else:
            raise ValueError("If fileName is not None must provide dpi argument")
    else:
        plt.show()



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

import math
from enum import Enum

from pycivil.EXAUtils.math import Fun, piecewiseFun


class RMinutes(str, Enum):
    R30 = "R30"
    R60 = "R60"
    R90 = "R90"
    R120 = "R120"
    R150 = "R150"
    R180 = "R180"
    RXXX = "RXXX"


class Moisture(str, Enum):
    v00 = "0.00 %"
    v15 = "1.50 %"
    v30 = "3.00 %"


class Aggregates(str, Enum):
    CALCAREOUS = "calcareous"
    SILICEOUS = "siliceous"


def concreteElongation(theta: float, aggregates: Aggregates) -> float:
    if aggregates == Aggregates.SILICEOUS:
        if 20 <= theta < 700:
            return -1.8e-04 + 9.0e-06 * theta + 2.3e-11 * theta * theta * theta
        elif 700 <= theta <= 1200:
            return 14.0e-03
        else:
            print("Theta greater than 1200 min or less than 20 !!!")
            return 0.0

    elif aggregates == Aggregates.CALCAREOUS:
        if 20 <= theta < 805:
            return -1.2e-04 + 6.0e-06 * theta + 1.4e-11 * theta * theta * theta
        elif 805 <= theta <= 1200:
            return 12.0e-03
        else:
            print("Theta greater than 1200 min or less than 20 !!!")
            return 0.0
    else:
        print("Aggregates unknown !!!")
        return 0.0


def capacity(theta: float, moisture: Moisture) -> float:
    if moisture == Moisture.v00:
        cPeak = 900
    elif moisture == Moisture.v15:
        cPeak = 1470
    elif moisture == Moisture.v30:
        cPeak = 2020
    else:
        print("Moisture unknown !!!")
        return 0.0

    if 20 <= theta and theta <= 100:
        return 900
    elif 100 <= theta and theta <= 115:
        return cPeak
    elif 115 <= theta and theta <= 200:
        return cPeak - (theta - 115) / 85 * (cPeak - 1000)
    elif 200 <= theta and theta <= 400:
        return 1000 + (theta - 200) / 2
    elif 400 <= theta and theta <= 1200:
        return 1100
    else:
        return 0.0


def ro(teta: float, ro20: float = 2300) -> float:
    if 20 <= teta <= 115:
        return ro20
    elif 115 <= teta <= 200:
        return ro20 * (1 - 0.02 * (teta - 115) / 85)
    elif 200 <= teta <= 400:
        return ro20 * (0.98 - 0.03 * (teta - 200) / 200)
    elif 400 <= teta <= 1200:
        return ro20 * (0.95 - 0.07 * (teta - 400) / 800)
    else:
        print("ERR: out of the range !!!")
        return 0.0


def capacityVolumic(
    theta: float, moisture: Moisture = Moisture.v00, ro20: float = 2300
) -> float:
    return ro(theta, ro20) * capacity(theta, moisture)


class LimitCurve(str, Enum):
    INF = "inf"
    SUP = "sup"


def cond(theta: float, limit: LimitCurve) -> float:
    if limit == LimitCurve.INF:
        A = 1.36
        B = 0.136
        C = 0.0057
    elif limit == LimitCurve.SUP:
        A = 2.00
        B = 0.2451
        C = 0.0107
    else:
        print("Limit curve unknown !!!")
        return 0.0

    return A - B * theta / 100 + C * (theta / 100) ** 2


class FireCurve(str, Enum):
    ISO834 = "ISO834"
    HC = "HC"
    HCM = "HCM"
    RWS = "RWS"


def tempByTime(t: float, curve: FireCurve) -> float:
    if curve == FireCurve.ISO834:
        return 20 + 345 * math.log10(8 * t + 1)
    elif curve == FireCurve.HC:
        return 20 + 1080 * (
            1 - 0.325 * math.exp(-0.167 * t) - 0.675 * math.exp(-2.5 * t)
        )
    elif curve == FireCurve.HCM:
        return 20 + 1280 * (
            1 - 0.325 * math.exp(-0.167 * t) - 0.675 * math.exp(-2.5 * t)
        )
    elif curve == FireCurve.RWS:
        if 0.0 <= t and t <= 3.0:
            return 290 * t + 20
        elif 3.0 <= t and t <= 5.0:
            return 125 * t + 515
        elif 5.0 <= t and t <= 10.0:
            return 12 * t + 1080
        elif 10.0 <= t and t <= 30.0:
            return 5 * t + 1150
        elif 30.0 <= t and t <= 60.0:
            return 5 / 3 * t + 1250
        elif 60.0 <= t and t <= 90.0:
            return -5 / 3 * t + 1450
        elif 90.0 <= t and t <= 120.0:
            return -10 / 3 * t + 1600
        elif 120.0 <= t and t <= 180.0:
            return 1200
        else:
            print("Time greater than 180 min !!!")
            return 0.0
    else:
        print("Fire curve unknown !!!")
        return 0.0


class SteelShapingType(str, Enum):
    HOT_ROLLED = "hot rolled"
    COLD_ROLLED = "cold rolled"


class StressType(Enum):
    TENSION = "tension"
    COMPRESSION = "compression"


def kappa_s(
    teta: float,
    shaping: SteelShapingType = SteelShapingType.HOT_ROLLED,
    stress: StressType = StressType.TENSION,
) -> float:

    if stress == StressType.COMPRESSION:
        if 20 <= teta and teta < 100:
            return 1.0
        elif 100 <= teta and teta < 400:
            return 0.7 - 0.3 * (teta - 400) / 300
        elif 400 <= teta and teta < 500:
            return 0.57 - 0.13 * (teta - 500) / 100
        elif 500 <= teta and teta < 700:
            return 0.10 - 0.47 * (teta - 700) / 200
        elif 700 <= teta and teta <= 1200:
            return 0.10 * (1200 - teta) / 500
        else:
            print("ERR: out of the range !!!")
        return 0.0
    else:
        if shaping == SteelShapingType.HOT_ROLLED:
            if 20 <= teta and teta < 400:
                return 1.0
            elif 400 <= teta and teta < 500:
                return -2.20e-03 * teta + 1.88
            elif 500 <= teta and teta < 600:
                return -3.10e-03 * teta + 2.33
            elif 600 <= teta and teta < 700:
                return -2.40e-03 * teta + 1.91
            elif 700 <= teta and teta < 800:
                return -1.20e-03 * teta + 1.07
            elif 800 <= teta and teta < 900:
                return -5.00e-04 * teta + 0.51
            elif 900 <= teta and teta <= 1200:
                return -2.00e-04 * teta + 0.24
            else:
                print("ERR: out of the range !!!")
            return 0.0

        elif shaping == SteelShapingType.COLD_ROLLED:
            if 20 <= teta and teta < 300:
                return 1.0
            elif 300 <= teta and teta < 400:
                return -6.00e-04 * teta + 1.18
            elif 400 <= teta and teta < 600:
                return -2.70e-03 * teta + 2.02
            elif 600 <= teta and teta < 700:
                return -2.80e-03 * teta + 2.08
            elif 700 <= teta and teta < 800:
                return -1.00e-04 * teta + 0.19
            elif 800 <= teta and teta < 1000:
                return -3.00e-04 * teta + 0.35
            elif 1000 <= teta and teta < 1100:
                return -2.00e-04 * teta + 0.25
            elif 1100 <= teta and teta <= 1200:
                return -3.00e-04 * teta + 0.36
            else:
                print("ERR: out of the range !!!")

            return 0.0

        else:
            print("ERR: SteelShapingType unknown !!!")

    return 0.0


class fck_theta_div_fck(piecewiseFun):
    def __init__(self, aggregates: Aggregates = Aggregates.SILICEOUS):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if aggregates == Aggregates.SILICEOUS:
            val = [
                1.0,
                1.0,
                0.95,
                0.85,
                0.75,
                0.60,
                0.45,
                0.30,
                0.15,
                0.08,
                0.04,
                0.01,
                0.00,
            ]
        else:
            val = [
                1.0,
                1.0,
                0.97,
                0.91,
                0.85,
                0.74,
                0.60,
                0.43,
                0.27,
                0.15,
                0.06,
                0.02,
                0.00,
            ]

        piecewiseFun.__init__(self, theta, val)


class ec1_theta(piecewiseFun):
    def __init__(self, aggregates: Aggregates = Aggregates.SILICEOUS):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if aggregates == Aggregates.SILICEOUS:
            val = [
                0.0025,
                0.0040,
                0.0055,
                0.0070,
                0.0100,
                0.0150,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
            ]
        else:
            val = [
                0.0025,
                0.0040,
                0.0055,
                0.0070,
                0.0100,
                0.0150,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
                0.025,
            ]
        piecewiseFun.__init__(self, theta, val)


class ecu1_theta(piecewiseFun):
    def __init__(self, aggregates: Aggregates = Aggregates.SILICEOUS):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if aggregates == Aggregates.SILICEOUS:
            val = [
                0.0200,
                0.0225,
                0.0250,
                0.0275,
                0.0300,
                0.0325,
                0.0350,
                0.0375,
                0.0400,
                0.0425,
                0.0450,
                0.0475,
                0.0500,
            ]
        else:
            val = [
                0.0200,
                0.0225,
                0.0250,
                0.0275,
                0.0300,
                0.0325,
                0.0350,
                0.0375,
                0.0400,
                0.0425,
                0.0450,
                0.0475,
                0.0500,
            ]
        piecewiseFun.__init__(self, theta, val)


class fc_theta_epsi(Fun):
    def __init__(
        self,
        fck: float,
        aggregates: Aggregates = Aggregates.SILICEOUS,
        theta: float = 20,
    ):
        super().__init__()
        self.__theta = theta
        self.__aggregates = aggregates
        self.__fck = fck

    @property
    def theta(self):
        return self.__theta

    @theta.setter
    def theta(self, val: float) -> None:
        if 20 <= val <= 1200:
            self.__theta = val
        else:
            raise ValueError("val must be in the field [20,1200]")

    @theta.deleter
    def theta(self):
        del self.__theta

    def eval(self, x: float) -> float:

        fc = fck_theta_div_fck(self.__aggregates).eval(self.__theta) * self.__fck
        ec1 = ec1_theta(self.__aggregates).eval(self.__theta)
        ecu1 = ecu1_theta(self.__aggregates).eval(self.__theta)

        if x >= 0 and x < ec1:
            return 3 * fc * x / (ec1 * (2 + (x / (ec1)) * (x / (ec1)) * (x / (ec1))))
        elif x >= ec1 and x < ecu1:
            return fc * (ecu1 - x) / (ecu1 - ec1)
        else:
            return 0.0


class fsy_theta_div_fyk(piecewiseFun):
    def __init__(self, formed: SteelShapingType = SteelShapingType.HOT_ROLLED):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if formed == SteelShapingType.HOT_ROLLED:
            val = [
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                0.78,
                0.47,
                0.23,
                0.11,
                0.06,
                0.04,
                0.02,
                0.00,
            ]
        else:
            val = [
                1.0,
                1.0,
                1.0,
                1.0,
                0.94,
                0.67,
                0.40,
                0.12,
                0.11,
                0.08,
                0.05,
                0.03,
                0.00,
            ]

        piecewiseFun.__init__(self, theta, val)


class fsp_theta_div_fyk(piecewiseFun):
    def __init__(self, formed: SteelShapingType = SteelShapingType.HOT_ROLLED):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if formed == SteelShapingType.HOT_ROLLED:
            val = [
                1.0,
                1.0,
                0.81,
                0.61,
                0.42,
                0.36,
                0.18,
                0.07,
                0.05,
                0.04,
                0.02,
                0.01,
                0.00,
            ]
        else:
            val = [
                1.0,
                0.96,
                0.92,
                0.81,
                0.63,
                0.44,
                0.26,
                0.08,
                0.06,
                0.05,
                0.03,
                0.02,
                0.00,
            ]

        piecewiseFun.__init__(self, theta, val)


class Es_theta_div_Es(piecewiseFun):
    def __init__(self, formed: SteelShapingType = SteelShapingType.HOT_ROLLED):
        theta = [
            20.0,
            100.0,
            200.0,
            300.0,
            400.0,
            500.0,
            600.0,
            700.0,
            800.0,
            900.0,
            1000.0,
            1100.0,
            1200.0,
        ]
        if formed == SteelShapingType.HOT_ROLLED:
            val = [
                1.00,
                1.00,
                0.90,
                0.80,
                0.70,
                0.60,
                0.31,
                0.13,
                0.09,
                0.07,
                0.04,
                0.02,
                0.00,
            ]
        else:
            val = [
                1.00,
                1.00,
                0.87,
                0.72,
                0.56,
                0.40,
                0.24,
                0.08,
                0.06,
                0.05,
                0.03,
                0.02,
                0.00,
            ]

        piecewiseFun.__init__(self, theta, val)


class fs_theta_epsi(Fun):
    def __init__(
        self,
        fyk: float,
        Es: float,
        formed: SteelShapingType = SteelShapingType.HOT_ROLLED,
        theta: float = 20,
    ):
        super().__init__()
        self.__theta = theta
        self.__formed = formed
        self.__fyk = fyk
        self.__Es = Es

    @property
    def theta(self):
        return self.__theta

    @theta.setter
    def theta(self, val: float) -> None:
        if 20 <= val <= 1200:
            self.__theta = val
        else:
            raise ValueError("val must be in the field [20,1200]")

    @theta.deleter
    def theta(self):
        del self.__theta

    def eval(self, x: float) -> float:

        Es_theta = Es_theta_div_Es(self.__formed).eval(self.__theta) * self.__Es
        fsy_theta = fsy_theta_div_fyk(self.__formed).eval(self.__theta) * self.__fyk
        fsp_theta = fsp_theta_div_fyk(self.__formed).eval(self.__theta) * self.__fyk

        esp_theta = fsp_theta / Es_theta
        esy_theta = 0.02
        est_theta = 0.15
        esu_theta = 0.20

        c = ((fsy_theta - fsp_theta) ** 2) / (
            (esy_theta - esp_theta) * Es_theta - 2 * (fsy_theta - fsp_theta)
        )
        a = math.sqrt((esy_theta - esp_theta) * (esy_theta - esp_theta + c / Es_theta))
        b = math.sqrt(c * (esy_theta - esp_theta) * Es_theta + c**2)

        if 0 <= x < esp_theta:
            return x * Es_theta
        elif esp_theta <= x < esy_theta:
            return fsp_theta - c + (b / a) * math.sqrt(a**2 - (esy_theta - x) ** 2)
        elif esy_theta <= x < est_theta:
            return fsy_theta
        elif est_theta <= x < esu_theta:
            return fsy_theta * (1 - (x - est_theta) / (esu_theta - est_theta))
        else:
            return 0.0



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pathlib import Path

import pytest

from pycivil.EXAStructural.cheatsheets.codeEC2Rules import (
    PlateMinimumArea,
    PlateMinimumAreaInput,
    SolverPlateMinRebar,
    PlateMinimumAreaLogs,
    PlateMinimumAreaOutput
)
from pycivil.EXAUtils import latexCheatSheets
from pycivil.EXAUtils.report import (
    Fragment,
    Reporter,
    ReportTemplateEnum,
    getTemplatesPath,
)

@pytest.mark.needLatex
def test_plates_minimum_area_default(tmp_path: Path):
    dataInput = PlateMinimumAreaInput()
    solver = SolverPlateMinRebar(inputModel=dataInput)

    assert solver.run()

    modelLogs = solver.getModelLogs()
    assert isinstance(modelLogs, PlateMinimumAreaLogs)

    modelOuts = solver.getModelOutput()
    assert isinstance(modelOuts, PlateMinimumAreaOutput)

    data = PlateMinimumArea(
        inputData=dataInput,
        logsData=modelLogs,
        outputData=modelOuts,
    )

    latexTemplatePath = getTemplatesPath()

    print(f"Latex Templates path for test is {latexTemplatePath}")
    fragBuilder = latexCheatSheets.PlateMinimumAreaCS(
        latexTemplatePath=latexTemplatePath, data=data
    )

    frag_title = Fragment()
    frag_title.add(line=r"\section{Area minima delle piastre secondo EC2}")
    frag = fragBuilder.buildFragment()

    reporter = Reporter(latexTemplatePath)

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_ENG_CAL")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_ENG_CAL.pdf").exists()
    assert tmp_path.joinpath("TEX_ENG_CAL.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_KOMA, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_KOMA")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_KOMA.pdf").exists()
    assert tmp_path.joinpath("TEX_KOMA.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_MAIN,
        main_file_name="sheet-PlateMinimumArea-EC211-ita-main.tex",
        fragments=[frag],
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_MAIN")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_MAIN.pdf").exists()
    assert tmp_path.joinpath("TEX_MAIN.tex").exists()



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pathlib import Path
from typing import List, Union, cast, Any, Dict

from pycivil.EXAUtils.EXAExceptions import EXAExceptions
from pycivil.EXAGeometry.shapes import ShapeArea, ShapeRect, ShapesEnum
from pycivil.EXAStructural.codes import KNOWN_CODES, Code, Codes
from pycivil.EXAStructural.loads import ForcesOnSection
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pycivil.EXAStructural.templateRCRect import RCTemplRectEC2
from pycivil.EXAUtils.report import EnumFBSection, Fragment, FragmentsBuilder


def texSpecChars(val: Union[str, None]) -> Union[str, None]:
    replaced = ""
    if val is not None:
        replaced = val.replace("_", "\\_")
    return replaced


class ForcesOnSectionListFB(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, forces: List[ForcesOnSection] | None = None):
        super().__init__()
        if forces is None:
            forces = []
        self.__forces = forces
        self.__opt_section_enabled = True
        self.__opt_section_title = "Forces on section lists with NTC2018"
        self.__opt_section_level = EnumFBSection.SEC_SUBSECTION
        self.__latexTemplatePath = latexTemplatePath

    def forces(self) -> List[ForcesOnSection]:
        return self.__forces

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        if "section_enabled" in options:
            self.__opt_section_enabled = options["section_enabled"]

        if "section_title" in options:
            self.__opt_section_title = options["section_title"]

        if "section_level" in options:
            self.__opt_section_level = options["section_level"]

        return True

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        if self.__opt_section_enabled:
            if self.__opt_section_level == EnumFBSection.SEC_CHAPTER:
                f.add(r"\chapter{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SECTION:
                f.add(r"\section{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSECTION:
                f.add(r"\subsection{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSUBSECTION:
                f.add(r"\subsubsection{" + self.__opt_section_title + "}")
            else:
                raise EXAExceptions(
                    "0001", "level number unknown", self.__opt_section_level
                )

        forces: List[Dict[str, str]] = []
        placeHolders = {"forces": forces}
        for fs in self.__forces:

            fd = {"id": f"{fs.id:.0f}"}

            comp = fs.Fx
            if isinstance(comp, (float, int)):
                fd["Fx"] = f"{comp / 1000:.1f}"
            else:
                fd["Fx"] = ""

            comp = fs.Fy
            if isinstance(comp, (float, int)):
                fd["Fy"] = f"{comp / 1000:.1f}"
            else:
                fd["Fy"] = ""

            comp = fs.Fz
            if isinstance(comp, (float, int)):
                fd["Fz"] = f"{comp / 1000:.1f}"
            else:
                fd["Fz"] = ""

            comp = fs.Mx
            if isinstance(comp, (float, int)):
                fd["Mx"] = f"{comp / 1000000:.1f}"
            else:
                fd["Mx"] = ""

            comp = fs.My
            if isinstance(comp, (float, int)):
                fd["My"] = f"{comp / 1000000:.1f}"
            else:
                fd["My"] = ""

            comp = fs.Mz
            if isinstance(comp, (float, int)):
                fd["Mz"] = f"{comp / 1000000:.1f}"
            else:
                fd["Mz"] = ""

            if fs.descr is not None:
                print(texSpecChars(fs.descr))
                fd["descr"] = f"{texSpecChars(fs.descr):s}"
            else:
                fd["descr"] = ""
            # fd["descr"] = f"{fs.descr:s}"
            fd["limitState"] = f"{fs.getLimitStateTr():s}"
            fd["frequency"] = f"{fs.getFrequencyTr():s}"
            forces.append(fd)

        f.add(
            templateName="template-ita-loads-NTC2018.tex",
            templatePlaceholders=placeHolders,
        )

        self._setFragment(f)
        return f


class CodesFB(Codes, FragmentsBuilder):
    def __init__(
        self, latexTemplatePath: Path, codes: Union[List[Code], None] = None
    ) -> None:

        if codes is None:
            codes = []

        Codes.__init__(self, codes)
        self.__opt_nameList = "Normative utilizzate"
        self.__opt_environment = True
        self.__opt_section = True
        self.__opt_subsection = False
        self.__latexTemplatePath = latexTemplatePath

    def setOptionNameList(self, name: str = "Normative utilizzate") -> None:
        self.__opt_nameList = name

    def setOptionSection(self) -> None:
        self.__opt_section = True
        self.__opt_subsection = False

    def setOptionSubSection(self) -> None:
        self.__opt_section = False
        self.__opt_subsection = True

    def setOptionEnvironnment(self, enabled: bool = True) -> None:
        self.__opt_environment = enabled

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        return True

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)

        if self.__opt_environment:
            if self.__opt_section:
                f.add(
                    r"\renewcommand{\bibsection}{\section{" + self.__opt_nameList + "}}"
                )
            elif self.__opt_subsection:
                f.add(
                    r"\renewcommand{\bibsection}{\subsection{"
                    + self.__opt_nameList
                    + "}}"
                )

            f.add(r"\begin{thebibliography}{99}")

        for c in self.getCodes():
            try:
                dfc = KNOWN_CODES[c.codeStr()]
            except KeyError:
                print("skipped")
                continue
            f.add(r"\bibitem{" + dfc.code + "}")
            f.add(
                r"\emph{"
                + dfc.description
                + "}"
                + r", \mbox{"
                + dfc.author
                + "}, "
                + str(dfc.year)
            )

        if self.__opt_environment:
            f.add(r"\end{thebibliography}")

        self._setFragment(f)
        return f


class ConcreteFB(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, concrete: Concrete):
        super().__init__()
        self.__concrete = concrete
        self.__opt_section_enabled = True
        self.__opt_section_title = "Concrete material"
        self.__opt_section_level = EnumFBSection.SEC_SUBSECTION
        self.__opt_glossary = False
        self.__latexTemplatePath = latexTemplatePath

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        if "section_enabled" in options:
            self.__opt_section_enabled = options["section_enabled"]

        if "section_title" in options:
            self.__opt_section_title = options["section_title"]

        if "section_level" in options:
            self.__opt_section_level = options["section_level"]

        if "glossary" in options:
            self.__opt_glossary = options["glossary"]

        return True

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        if self.__opt_section_enabled:
            if self.__opt_section_level == EnumFBSection.SEC_CHAPTER:
                f.add(r"\chapter{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SECTION:
                f.add(r"\section{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSECTION:
                f.add(r"\subsection{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSUBSECTION:
                f.add(r"\subsubsection{" + self.__opt_section_title + "}")
            else:
                raise EXAExceptions(
                    "0001", "level number unknown", self.__opt_section_level
                )

        placeHolders = {
            "classe": f"{self.__concrete.catStr():s}",
            "norma": f"{self.__concrete.codeStr():s}",
            "environnment": "{:s}".format(self.__concrete.getEnvironmentTr("IT")),
            "fck": "{:.1f} {:s}".format(self.__concrete.get_fck(), r"\text{ MPa}"),
            "fcm": "{:.1f} {:s}".format(self.__concrete.get_fcm(), r"\text{ MPa}"),
            "fctm": "{:.1f} {:s}".format(self.__concrete.get_fctm(), r"\text{ MPa}"),
            "Ecm": "{:.1f} {:s}".format(self.__concrete.get_Ecm(), r"\text{ MPa}"),
            "gammac": f"{self.__concrete.get_gammac():.2f}",
            "sigmac_max_c": "{:.1f} {:s}".format(
                self.__concrete.get_sigmac_max_c(), r"\text{ MPa}"
            ),
            "sigmac_max_q": "{:.1f} {:s}".format(
                self.__concrete.get_sigmac_max_q(), r"\text{ MPa}"
            ),
            "ecu": f"{self.__concrete.get_ecu():.4f}",
            "ec2": f"{self.__concrete.get_ec2():.4f}",
            "lambda": f"{self.__concrete.get_lambda():.2f}",
            "eta": f"{self.__concrete.get_eta():.2f}",
            "alphacc": f"{self.__concrete.get_alphacc():.2f}",
            "alphacc_fire": f"{self.__concrete.get_alphacc_fire():.2f}",
            "byCode": self.__concrete.isSetByCode(),
            "glossary": self.__opt_glossary,
        }

        f.add(
            templateName="template-ita-material-concrete.tex",
            templatePlaceholders=placeHolders,
        )

        self._setFragment(f)
        return f


class SteelConcreteFB(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, steel: ConcreteSteel):
        super().__init__()
        self.__steel = steel
        self.__opt_section_enabled = True
        self.__opt_section_title = "Steel for rebar material"
        self.__opt_section_level = EnumFBSection.SEC_SUBSECTION
        self.__latexTemplatePath = latexTemplatePath

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        if "section_enabled" in options:
            self.__opt_section_enabled = options["section_enabled"]

        if "section_title" in options:
            self.__opt_section_title = options["section_title"]

        if "section_level" in options:
            self.__opt_section_level = options["section_level"]

        return True

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        if self.__opt_section_enabled:
            if self.__opt_section_level == EnumFBSection.SEC_CHAPTER:
                f.add(r"\chapter{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SECTION:
                f.add(r"\section{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSECTION:
                f.add(r"\subsection{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSUBSECTION:
                f.add(r"\subsubsection{" + self.__opt_section_title + "}")
            else:
                raise EXAExceptions(
                    "0001", "level number unknown", self.__opt_section_level
                )

        placeHolders = {
            "classe": f"{self.__steel.catStr():s}",
            "norma": f"{self.__steel.codeStr():s}",
            "sensitivity": "{:s}".format(self.__steel.getEnvironmentTr("IT")),
            "fsy": "{:.0f} {:s}".format(self.__steel.get_fsy(), r"\text{ MPa}"),
            "Es": "{:.0f} {:s}".format(self.__steel.get_Es(), r"\text{ MPa}"),
            "gammas": f"{self.__steel.get_gammas():.2f}",
            "sigmas_max_c": "{:.1f} {:s}".format(
                self.__steel.get_sigmas_max_c(), r"\text{ MPa}"
            ),
            "esu": f"{self.__steel.get_esu():.4f}",
            "esy": f"{self.__steel.get_esy():.4f}",
            "byCode": self.__steel.isSetByCode(),
        }

        f.add(
            templateName="template-ita-material-rebar.tex",
            templatePlaceholders=placeHolders,
        )

        self._setFragment(f)
        return f


class ConcreteSectionFB(FragmentsBuilder):
    def __init__(self, latexTemplatePath: Path, rcs: RCTemplRectEC2):
        super().__init__()
        self.__rcs = rcs
        self.__opt_section_enabled = True
        self.__opt_section_title = "Reinforced concrete section"
        self.__opt_section_level = EnumFBSection.SEC_SUBSECTION
        self.__opt_glossary = False
        self.__latexTemplatePath = latexTemplatePath

    def getOptSectionEnabled(self) -> bool:
        return self.__opt_section_enabled

    def getOptSectionTitle(self) -> str:
        return self.__opt_section_title

    def getOptSectionLevel(self):
        return self.__opt_section_level

    def getOptGlossary(self) -> bool:
        return self.__opt_glossary

    def setFragmentOptions(self, options: Dict[str, Any]) -> bool:
        if "section_enabled" in options:
            self.__opt_section_enabled = options["section_enabled"]

        if "section_title" in options:
            self.__opt_section_title = options["section_title"]

        if "section_level" in options:
            self.__opt_section_level = options["section_level"]

        if "glossary" in options:
            self.__opt_glossary = options["glossary"]

        return True

    def buildFragment(self) -> Fragment:
        f = Fragment(self.__latexTemplatePath)
        if self.__opt_section_enabled:
            if self.__opt_section_level == EnumFBSection.SEC_CHAPTER:
                f.add(r"\chapter{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SECTION:
                f.add(r"\section{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSECTION:
                f.add(r"\subsection{" + self.__opt_section_title + "}")
            elif self.__opt_section_level == EnumFBSection.SEC_SUBSUBSECTION:
                f.add(r"\subsubsection{" + self.__opt_section_title + "}")
            else:
                raise EXAExceptions(
                    "0001", "level number unknown", self.__opt_section_level
                )

        placeHolders: Dict[str, Any] = {}
        #
        # RECTANGULAR SHAPE
        #
        if (
            self.__rcs.getStructConcretelItem().getShape().getType()
            == ShapesEnum.SHAPE_RECT
        ):
            placeHolders["shape"] = "RECT"
            #
            rectShape = cast(ShapeRect, self.__rcs.getStructConcretelItem().getShape())
            placeHolders["width"] = rectShape.w()
            placeHolders["height"] = rectShape.h()
            dimRef = max(rectShape.w(), rectShape.h())

            # Below value is centimeters
            figureRayDimension = 8
            dimLineDistance = 0.6

            scaleFigure = figureRayDimension / dimRef
            placeHolders["scale"] = scaleFigure
            placeHolders["dimDist"] = dimLineDistance / scaleFigure

            #
            placeHolders["xTL"] = rectShape.getShapePoint("TL").x
            placeHolders["xTR"] = rectShape.getShapePoint("TR").x
            placeHolders["xBL"] = rectShape.getShapePoint("BL").x
            placeHolders["xBR"] = rectShape.getShapePoint("BR").x
            placeHolders["yTL"] = rectShape.getShapePoint("TL").y
            placeHolders["yTR"] = rectShape.getShapePoint("TR").y
            placeHolders["yBL"] = rectShape.getShapePoint("BL").y
            placeHolders["yBR"] = rectShape.getShapePoint("BR").y
            #
        else:
            raise EXAExceptions("0001", "Section Unknown")

        # steelShape = typing.cast(eg.ShapeRect,self.getStructSteelItems().getShape())
        steels: List[Dict[str, str]] = []

        for idx, i in enumerate(self.__rcs.getStructSteelItems()):
            steel = {}
            areaShape = cast(ShapeArea, i.getShape())
            steel["id"] = i.getId() if i.getId() != -1 else ""
            steel["xPos"] = "{:.1f}".format(areaShape.getShapePoint("O").x)
            steel["yPos"] = "{:.1f}".format(areaShape.getShapePoint("O").y)
            steel["diam"] = f"{self.__rcs.getSteelDiamAt(idx):.0f}"
            steel["area"] = f"{self.__rcs.getSteelAreaAt(idx):.1f}"
            steels.append(steel)

        placeHolders["rebars"] = steels

        f.add(
            templateName="template-ita-rc-section-rec.tex",
            templatePlaceholders=placeHolders,
        )
        self._setFragment(f)
        return f



# Copyright (c) 2026 Luigi Paone <ppc.luigi.paone@gmail.com>
# SPDX-License-Identifier: BSD-3-Clause

from pycivil.EXAStructural import templateRCRect as est

# Setting new instance of section with
# id = 1 and name = "First Section"
section = est.RCTemplRectEC2(1, "First Section")

section.setDimH(500.0)
section.setDimW(300.0)

# 'MB' means medium bottom area
# 'MT' means medium top area
section.addSteelArea("MB", 40.0, 600.0)
section.addSteelArea("MT", 40.0, 600.0)

section.setMaterials("C32/40", "B450C")

# Adding Tension Points
KN = 1000
KNm = 1000000
section.addTensionPoint2d(N=500.0 * KN, M=191.2 * KNm)
section.addTensionPoint2d(N=0.0 * KN, M=104.0 * KNm)

nb = 30

# Building a set points of interaction diagramm
section.interactionDomainBuild2d(nbPoints=nb)

# Adding new Area
section.addSteelArea("MB", 40.0, 600.0)
section.addSteelArea("MT", 40.0, 600.0)


# Building a new points set of interaction diagramm
section.interactionDomainBuild2d(nbPoints=nb, SLS=False)
section.interactionDomainBuild2d(nbPoints=nb, SLS=True)
section.interactionDomainBuild2d(nbPoints=nb, alpha=2.0)
section.interactionDomainBuild2d(nbPoints=nb, alpha=3.0)

section.interactionDomainPlot2d(xLabel="N [KN]", yLabel="M [KN*m]")

print(section)
